@@ -630,6 +630,7 @@ int audio_write(uint8_t *buf, int size, int swap)
  * The audio task
  */
 
+#ifdef HAVE_SYS_SOUNDCARD_H
 static void system_error(const char *str, int fd, int use_strerror)
 {
    if(use_strerror)
@@ -646,6 +647,7 @@ static void system_error(const char *str, int fd, int use_strerror)
 	  pthread_exit(NULL);
 #endif
 }
+#endif /* HAVE_SYS_SOUNDCARD_H */
 
 #ifdef HAVE_SYS_SOUNDCARD_H
 void do_audio(void)
@@ -1090,9 +1092,6 @@ void do_audio(void)
 #else
 void do_audio()
 {
-  fprintf(stderr, "ILLEGAL CALL TO do_audio in audiolib.c: NOT IMPLEMENTED IN IRIX !\n");
+  fprintf(stderr, "Unsupported audio system in audiolib.c: no soundcard.hn");
 }
-
 #endif
-
-
