@@ -17,7 +17,7 @@
 #ifndef __AUDIOLIB_H__
 #define __AUDIOLIB_H__
 
-#include "mjpeg_types.h"
+#include <mjpeg_types.h>
 
 void audio_shutdown(void);
 
