@@ -49,7 +49,8 @@ static int pid;
 
 static double fps;
 static int cur_pos, total_frames, cur_speed=1, old_speed=999999;
-static int slider_pause = 0;
+static int slider_pause;
+static int slider_pos;
 
 static int ff_stat=0, fr_stat=0;
 static int ff_speed[4] = { 1, 3, 10, 30 };
@@ -65,7 +66,6 @@ static int savetype = 0;
 static char inpbuff[MAXINP];
 static int inplen = 0;
 
-static int hscale_down = 0;
 static int frame_skip_button_up = 1;
 static char frame_skip_char;
 
@@ -92,7 +92,9 @@ static void calc_timecode(int pos, int do_frames)
 static void store_filename(GtkFileSelection *selector, gpointer user_data) {
    char str[256];
    const char *name;
-   selected_filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(file_selector));
+
+   /* First easy fix, Bernhard */
+   selected_filename = (char*)gtk_file_selection_get_filename (GTK_FILE_SELECTION(file_selector));
    name = selected_filename;
    if(name==0) return;
    switch(savetype) {
@@ -114,14 +116,18 @@ static void create_file_selection(void) {
       default: printf("Error in create_file_selection\n"); return; break;
    }
    file_selector = gtk_file_selection_new(label);
-   gtk_signal_connect (GTK_OBJECT(GTK_FILE_SELECTION(file_selector)->ok_button),
-               "clicked", GTK_SIGNAL_FUNC (store_filename), NULL);
+
+   /* Next problem, Bernhard */
+   g_signal_connect_swapped(G_OBJECT(GTK_FILE_SELECTION
+                           (file_selector)->ok_button),  "clicked", 
+                           (GtkSignalFunc)store_filename, NULL);
    /* Ensure that the dialog box is destroyed when the user clicks a button. */
-   gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->ok_button),
-                    "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
+   g_signal_connect_swapped(G_OBJECT (GTK_FILE_SELECTION(file_selector)->ok_button),
+                    "clicked", G_CALLBACK (gtk_widget_destroy),
                     (gpointer) file_selector);
-   gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->cancel_button),
-                    "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
+
+   g_signal_connect_swapped(G_OBJECT (GTK_FILE_SELECTION(file_selector)->cancel_button),
+                    "clicked", G_CALLBACK (gtk_widget_destroy),
                     (gpointer) file_selector);
    
    /* Display that dialog */
@@ -132,30 +138,45 @@ gint key_press_cb(GtkWidget * widget, GdkEventKey* event, gpointer data )
 {
    int need_pause=FALSE;
    int n=1;
-   hscale_down=1;
    switch (event->keyval) {
       case GDK_0:/* go to beginning of file */
-         gtk_adjustment_set_value(GTK_ADJUSTMENT(gtk_xlav->timeslider),(gfloat)0);
+         write(out_pipe,"s0\n",3); /* go to beginning */
          break;
       case GDK_9: /* go to end of file */
          need_pause=TRUE;
-         write(out_pipe,"s10000000\n",10); break;  /* go to end */
+         write(out_pipe,"s10000000\n",10); /* go to end */
          break;
       case GDK_parenleft: /* mark start of selection */
       case GDK_Home:
-         gtk_signal_emit_by_name(GTK_OBJECT(gtk_xlav->BSSelStart),"clicked",(gpointer)1);
+         g_signal_emit_by_name(G_OBJECT(gtk_xlav->BSSelStart),"clicked",(gpointer)1);
          break;
       case GDK_parenright: /* mark end of selection */
       case GDK_End:
-         gtk_signal_emit_by_name(GTK_OBJECT(gtk_xlav->BSSelEnd),"clicked",(gpointer)1);
+         g_signal_emit_by_name(G_OBJECT(gtk_xlav->BSSelEnd),"clicked",(gpointer)1);
+         break;
+      case GDK_bracketleft: /* skip to selected start */
+         g_signal_emit_by_name(G_OBJECT(gtk_xlav->BGotoSelStart),"clicked",(gpointer)1);
+         break;
+      case GDK_bracketright: /* skip to selected end */
+         g_signal_emit_by_name(G_OBJECT(gtk_xlav->BGotoSelEnd),"clicked",(gpointer)1);
+         break;
+      case GDK_a: /* Save all */
+         g_signal_emit_by_name(G_OBJECT(gtk_xlav->BSaveAll),"clicked",(gpointer)1);
+         break;
+      case GDK_v: /* Save */
+         g_signal_emit_by_name(G_OBJECT(gtk_xlav->BSaveSel),"clicked",(gpointer)1);
+         break;
+      case GDK_q: /* Exit */
+         g_signal_emit_by_name(G_OBJECT(gtk_xlav->Exit),"clicked",(gpointer)1);
          break;
       case GDK_l: /* some number of frames right */
+      case GDK_L:
       case GDK_Right:
          need_pause=TRUE;
-         gtk_signal_emit_stop_by_name(GTK_OBJECT(gtk_xlav->xlav), "key_press_event");
+         g_signal_stop_emission_by_name(G_OBJECT(gtk_xlav->xlav), "key_press_event");
          n=1;
          if(event->state & GDK_CONTROL_MASK) {
-            write(out_pipe,"s10000000\n",10); break;  /* go to end */
+            write(out_pipe,"s10000000\n",10); /* go to end */
             break;
          } else if(event->state & GDK_MOD1_MASK) {
             n=50;
@@ -165,11 +186,13 @@ gint key_press_cb(GtkWidget * widget, GdkEventKey* event, gpointer data )
          skip_num_frames(n);
          break;
       case GDK_h: /* some number of frames left */
+      case GDK_H:
       case GDK_Left:
-         gtk_signal_emit_stop_by_name(GTK_OBJECT(gtk_xlav->xlav), "key_press_event");
+         g_signal_stop_emission_by_name(GTK_OBJECT(gtk_xlav->xlav), "key_press_event");
          n=-1;
          if(event->state & GDK_CONTROL_MASK) { /* go to beginning  */
-            gtk_adjustment_set_value(GTK_ADJUSTMENT(gtk_xlav->timeslider),(gfloat)0);
+            write(out_pipe,"s0\n",3); /* go to beginning */
+	    break;
          } else if(event->state & GDK_MOD1_MASK) {
             n=-50;
          } else if (event->state & GDK_SHIFT_MASK) { 
@@ -194,32 +217,35 @@ gint key_press_cb(GtkWidget * widget, GdkEventKey* event, gpointer data )
          need_pause=TRUE;
          skip_num_frames(-30);
          break;
+      case GDK_c: /* clear selection */
+         g_signal_emit_by_name(G_OBJECT(gtk_xlav->BClearSel),"clicked",(gpointer)1);
+         break;
       case GDK_x: /* cut selection */
       case GDK_Delete:
-         gtk_signal_emit_by_name(GTK_OBJECT(gtk_xlav->BECut),"clicked",(gpointer)1);
+         g_signal_emit_by_name(G_OBJECT(gtk_xlav->BECut),"clicked",(gpointer)1);
          break;
       case GDK_y: /* copy (yank) selection */
-         gtk_signal_emit_by_name(GTK_OBJECT(gtk_xlav->BECopy),"clicked",(gpointer)1);
+         g_signal_emit_by_name(G_OBJECT(gtk_xlav->BECopy),"clicked",(gpointer)1);
          break;
       case GDK_p: /* paste selection */
       case GDK_Insert:
-         gtk_signal_emit_by_name(GTK_OBJECT(gtk_xlav->BEPaste),"clicked",(gpointer)1);
+         g_signal_emit_by_name(G_OBJECT(gtk_xlav->BEPaste),"clicked",(gpointer)1);
          break;
       case GDK_f: /*  play (forward) */
-         gtk_signal_emit_by_name(GTK_OBJECT(gtk_xlav->play),"clicked",(gpointer)4);
+         g_signal_emit_by_name(G_OBJECT(gtk_xlav->play),"clicked",(gpointer)4);
          break;
       case GDK_F: /* fast forward */
-         gtk_signal_emit_by_name(GTK_OBJECT(gtk_xlav->ff),"clicked",(gpointer)5);
+         g_signal_emit_by_name(G_OBJECT(gtk_xlav->ff),"clicked",(gpointer)5);
          break;
       case GDK_r: /* play reverse */
-         gtk_signal_emit_by_name(GTK_OBJECT(gtk_xlav->rew),"clicked",(gpointer)2);
+         g_signal_emit_by_name(G_OBJECT(gtk_xlav->rew),"clicked",(gpointer)2);
          break;
       case GDK_R: /* fast reverse */
-         gtk_signal_emit_by_name(GTK_OBJECT(gtk_xlav->fr),"clicked",(gpointer)1);
+         g_signal_emit_by_name(G_OBJECT(gtk_xlav->fr),"clicked",(gpointer)1);
          break;
       case GDK_s: /* stop */
       case GDK_S:
-         gtk_signal_emit_by_name(GTK_OBJECT(gtk_xlav->stop),"clicked",(gpointer)3);
+         g_signal_emit_by_name(G_OBJECT(gtk_xlav->stop),"clicked",(gpointer)3);
          break;
       case GDK_1: /* go 5 seconds forward */
          skip_num_frames(150);
@@ -268,7 +294,6 @@ gint key_press_cb(GtkWidget * widget, GdkEventKey* event, gpointer data )
          write(out_pipe,"p0\n",3); /* pause on all keys */
       }
    }
-   hscale_down=0;
    return 0;
 }
 
@@ -284,8 +309,8 @@ static void quick_message(const char *message) {
    
    /* Ensure that the dialog box is destroyed when the user clicks ok. */
    
-   gtk_signal_connect_object (GTK_OBJECT (okay_button), "clicked",
-                              GTK_SIGNAL_FUNC (gtk_widget_destroy), GTK_OBJECT(dialog));
+   g_signal_connect_swapped(G_OBJECT (okay_button), "clicked",
+                         G_CALLBACK (gtk_widget_destroy), G_OBJECT(dialog));
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
                       okay_button);
 
@@ -308,18 +333,17 @@ void dispatch_input(void)
 
    if(inpbuff[0]=='@')
    {
+      int slider_new_pos;
       sscanf(inpbuff+1,"%lg/%d/%d/%d",&fps,&cur_pos,&total_frames,&cur_speed);
       calc_timecode(cur_pos,cur_speed==0);
       gtk_label_set_text(GTK_LABEL(gtk_xlav->Timer),timecode);
       /* fl_set_object_label(gtk_xlav->Timer,timecode); */
       if(total_frames<1) total_frames=1;
-      if(slider_pause) {
-         slider_pause--;
-
-      } else {
-         if (!hscale_down) {
-         gtk_adjustment_set_value(GTK_ADJUSTMENT(gtk_xlav->timeslider),(100.0)*(double) ((double)cur_pos)/((double)total_frames));
-         }
+      slider_new_pos = 10000 * cur_pos / total_frames;
+      if (slider_pos != slider_new_pos) {
+	 slider_pos = slider_new_pos;
+	 slider_pause++;
+	 gtk_adjustment_set_value(GTK_ADJUSTMENT(gtk_xlav->timeslider), (double)slider_new_pos / 100.0);
       }
       if(cur_speed != old_speed) {
          char label[32];
@@ -368,28 +392,18 @@ static void get_input(gpointer data, gint fd, GdkInputCondition condition)
    }
 }
 
-
-void        timehscale_button_pressed_cb(GtkAdjustment *adjustment, gpointer data) {
-   hscale_down=1;
-}
-
-void        timehscale_button_released_cb(GtkAdjustment *adjustment, gpointer data) {
-   hscale_down=0;
-}
-
 void        timeslider_cb(GtkAdjustment *adjustment, gpointer data)
 {
-   gfloat val;
-   char out[256];
+gfloat val;
+char out[256];
 
-   if (hscale_down) {
-      float new;
+   if (0 < slider_pause) {
+      --slider_pause;
+   } else {
       val = ((GTK_ADJUSTMENT(gtk_xlav->timeslider)->value));
-      new = (val / 100.00 );
       sprintf(out,"s%d\n",(int)((val*total_frames)/100));
       write(out_pipe,out,strlen(out));
    }
-      slider_pause = 8;
 }
 
 void button_cb(GtkWidget *ob, long data)
@@ -681,8 +695,10 @@ int main(int argc, char *argv[])
 
    gtk_init (&argc, &argv);
    gtk_xlav = create_form_xlav();
-   gtk_signal_connect (GTK_OBJECT (gtk_xlav->xlav), "destroy",
-       GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
+// old:  gtk_signal_connect (GTK_OBJECT (gtk_xlav->xlav), "destroy",
+// old:      GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
+   g_signal_connect_swapped(G_OBJECT (gtk_xlav->xlav), "destroy",
+   G_CALLBACK(gtk_main_quit), NULL);
    gtk_widget_show(gtk_xlav->xlav); /* show the main window */
 
    gdk_input_add(inp_pipe,GDK_INPUT_READ,(GdkInputFunction)get_input,(gpointer)0);
