@@ -46,7 +46,7 @@
  *                  1: Interlaced, Top field first
  *                  2: Interlaced, Bottom field first
  * ctype            Chroma format for decompression.
- *                  Currently always 420 and hence ignored.
+ *                  Currently only CHROMA420 and CHROMA422 are available
  * raw0             buffer with input / output raw Y channel
  * raw1             buffer with input / output raw U/Cb channel
  * raw2             buffer with input / output raw V/Cr channel
@@ -276,6 +276,10 @@ jpeg_buffer_dest (j_compress_ptr cinfo, unsigned char *buf, long len)
 struct my_error_mgr {
    struct jpeg_error_mgr pub;   /* "public" fields */
    jmp_buf setjmp_buffer;       /* for return to caller */
+
+	/* original emit_message method */
+   JMETHOD(void, original_emit_message, (j_common_ptr cinfo, int msg_level));
+   int warning_seen;		/* was a corrupt-data warning seen */
 };
 
 static void my_error_exit (j_common_ptr cinfo)
@@ -291,6 +295,18 @@ static void my_error_exit (j_common_ptr cinfo)
    longjmp (myerr->setjmp_buffer, 1);
 }
 
+static void my_emit_message (j_common_ptr cinfo, int msg_level)
+{
+   /* cinfo->err really points to a my_error_mgr struct, so coerce pointer */
+   struct my_error_mgr *myerr = (struct my_error_mgr *) cinfo->err;
+
+   if(msg_level < 0)
+	   myerr->warning_seen = 1;
+
+   /* call original emit_message() */
+   (myerr->original_emit_message)(cinfo, msg_level);
+}
+
 #define MAX_LUMA_WIDTH   4096
 #define MAX_CHROMA_WIDTH 2048
 
@@ -431,7 +447,13 @@ static void guarantee_huff_tables(j_decompress_ptr dinfo)
  *                  1: Interlaced, Top field first
  *                  2: Interlaced, Bottom field first
  * ctype            Chroma format for decompression.
- *                  Currently always 420 and hence ignored.
+ *                  Currently only CHROMA420 and CHROMA422 are available
+ * returns:
+ *	-1 on fatal error
+ *	0 on success
+ *	1 if jpeg lib threw a "corrupt jpeg data" warning.  
+ *		in this case, "a damaged output image is likely."
+ *	
  */
 
 int decode_jpeg_raw (unsigned char *jpeg_data, int len,
@@ -459,6 +481,10 @@ int decode_jpeg_raw (unsigned char *jpeg_data, int len,
    /* We set up the normal JPEG error routines, then override error_exit. */
    dinfo.err = jpeg_std_error (&jerr.pub);
    jerr.pub.error_exit = my_error_exit;
+   /* also hook the emit_message routine to note corrupt-data warnings */
+   jerr.original_emit_message = jerr.pub.emit_message;
+   jerr.pub.emit_message = my_emit_message;
+   jerr.warning_seen = 0;
 
    /* Establish the setjmp return context for my_error_exit to use. */
    if (setjmp (jerr.setjmp_buffer)) {
@@ -648,29 +674,62 @@ int decode_jpeg_raw (unsigned char *jpeg_data, int len,
                }
          }
 
-	 /* Vertical downsampling of chroma */
+	 /* Vertical resampling of chroma */
 
-         if (vsf[0] == 1) {
-	   /* Really downsample */
-	   for (y = 0; y < 8 /*&& yc < height/2*/; y += 2, yc += numfields) {
-               xd = yc * width / 2;
-               for (x = 0; x < width / 2; x++, xd++) {
+	 switch (ctype) {
+	 case CHROMA422:
+	   if (vsf[0] == 1) {
+	     /* Just copy */
+	     for (y = 0; y < 8 /*&& yc < height */; y++, yc += numfields) {
+	       xd = yc * width / 2;
+	       for (x = 0; x < width / 2; x++, xd++) {
+		 raw1[xd] = chr1[y][x];
+		 raw2[xd] = chr2[y][x];
+	       }
+	     }
+	   } else {
+	     /* upsample */
+	     for (y = 0; y < 8 /*&& yc < height */; y++) {
+	       xd = yc * width / 2;
+	       for (x = 0; x < width / 2; x++, xd++) {
+		 raw1[xd] = chr1[y][x];
+		 raw2[xd] = chr2[y][x];
+	       }
+	       yc += numfields;
+	       xd = yc * width / 2;
+	       for (x = 0; x < width / 2; x++, xd++) {
+		 raw1[xd] = chr1[y][x];
+		 raw2[xd] = chr2[y][x];
+	       }
+	       yc += numfields;
+	     }
+	   }
+	   break;
+	 default:
+	 /* should be case CHROMA420: but use default: for compatibility of liblavjpeg.so */
+	   if (vsf[0] == 1) {
+	     /* Really downsample */
+	     for (y = 0; y < 8 /*&& yc < height/2*/; y += 2, yc += numfields) {
+	       xd = yc * width / 2;
+	       for (x = 0; x < width / 2; x++, xd++) {
 		 assert(xd < (width * height / 4));
 		 raw1[xd] = (chr1[y][x] + chr1[y + 1][x]) >> 1;
 		 raw2[xd] = (chr2[y][x] + chr2[y + 1][x]) >> 1;
-               }
-            }
+	       }
+	     }
 
-         } else {
-	   /* Just copy */
-	   for (y = 0; y < 8 /*&& yc < height/2 */; y++, yc += numfields) {
-               xd = yc * width / 2;
-               for (x = 0; x < width / 2; x++, xd++) {
+	   } else {
+	     /* Just copy */
+	     for (y = 0; y < 8 /*&& yc < height/2 */; y++, yc += numfields) {
+	       xd = yc * width / 2;
+	       for (x = 0; x < width / 2; x++, xd++) {
 		 raw1[xd] = chr1[y][x];
-                 raw2[xd] = chr2[y][x];
-               }
-            }
-         }
+		 raw2[xd] = chr2[y][x];
+	       }
+	     }
+	   }
+	   break;
+	 }
       }
 
       (void) jpeg_finish_decompress (&dinfo);
@@ -689,7 +748,10 @@ int decode_jpeg_raw (unsigned char *jpeg_data, int len,
      }
 
    jpeg_destroy_decompress (&dinfo);
-   return 0;
+   if(jerr.warning_seen)
+	   return 1;
+   else
+	   return 0;
 
  ERR_EXIT:
    jpeg_destroy_decompress (&dinfo);
@@ -703,7 +765,7 @@ int decode_jpeg_raw (unsigned char *jpeg_data, int len,
  *                  1: Interlaced, Top field first
  *                  2: Interlaced, Bottom field first
  * ctype            Chroma format for decompression.
- *                  Currently always 420 and hence ignored.
+ *                  Currently only CHROMA420 and CHROMA422 are available
  */
 
 
@@ -883,26 +945,59 @@ int decode_jpeg_gray_raw (unsigned char *jpeg_data, int len,
 
          //mjpeg_info("/* Vertical downsampling of chroma, line %d, max %d */", dinfo.output_scanline, dinfo.output_height);
 
-         if (vsf[0] == 1) {
-            /* Really downsample */
-            for (y = 0; y < 8; y += 2, yc += numfields) {
-               xd = yc * width / 2;
-               for (x = 0; x < width / 2; x++, xd++) {
+	 switch (ctype) {
+	 case CHROMA422:
+	   if (vsf[0] == 1) {
+	     /* Just copy */
+	     for (y = 0; y < 8 /*&& yc < height */; y++, yc += numfields) {
+	       xd = yc * width / 2;
+	       for (x = 0; x < width / 2; x++, xd++) {
+		 raw1[xd] = 127; //chr1[y][x];
+		 raw2[xd] = 127; //chr2[y][x];
+	       }
+	     }
+	   } else {
+	     /* upsample */
+	     for (y = 0; y < 8 /*&& yc < height */; y++) {
+	       xd = yc * width / 2;
+	       for (x = 0; x < width / 2; x++, xd++) {
+		 raw1[xd] = 127; //chr1[y][x];
+		 raw2[xd] = 127; //chr2[y][x];
+	       }
+	       yc += numfields;
+	       xd = yc * width / 2;
+	       for (x = 0; x < width / 2; x++, xd++) {
+		 raw1[xd] = 127; //chr1[y][x];
+		 raw2[xd] = 127; //chr2[y][x];
+	       }
+	       yc += numfields;
+	     }
+	   }
+	   break;
+	 default:
+	 /* should be case CHROMA420: but use default: for compatibility of liblavjpeg.so */
+	   if (vsf[0] == 1) {
+	     /* Really downsample */
+	     for (y = 0; y < 8; y += 2, yc += numfields) {
+	       xd = yc * width / 2;
+	       for (x = 0; x < width / 2; x++, xd++) {
 		 raw1[xd] = 127; //(chr1[y][x] + chr1[y + 1][x]) >> 1;
-                 raw2[xd] = 127; //(chr2[y][x] + chr2[y + 1][x]) >> 1;
-               }
-            }
-         } else {
-            /* Just copy */
+		 raw2[xd] = 127; //(chr2[y][x] + chr2[y + 1][x]) >> 1;
+	       }
+	     }
+	   } else {
+	     /* Just copy */
 
-            for (y = 0; y < 8; y++, yc += numfields) {
-               xd = yc * width / 2;
-               for (x = 0; x < width / 2; x++, xd++) {
-                 raw1[xd] = 127; //chr1[y][x];
-                 raw2[xd] = 127; //chr2[y][x];
-               }
-            }
-         }
+	     for (y = 0; y < 8; y++, yc += numfields) {
+	       xd = yc * width / 2;
+	       for (x = 0; x < width / 2; x++, xd++) {
+		 raw1[xd] = 127; //chr1[y][x];
+		 raw2[xd] = 127; //chr2[y][x];
+	       }
+	     }
+	   }
+	   break;
+	 }
       }
 
       (void) jpeg_finish_decompress (&dinfo);
@@ -933,7 +1028,7 @@ int decode_jpeg_gray_raw (unsigned char *jpeg_data, int len,
  *                  1: Interlaced, Top field first
  *                  2: Interlaced, Bottom field first
  * ctype            Chroma format for decompression.
- *                  Currently always 420 and hence ignored.
+ *                  Currently only CHROMA420 and CHROMA422 are available
  */
 
 int encode_jpeg_raw (unsigned char *jpeg_data, int len, int quality,
@@ -1053,7 +1148,8 @@ int encode_jpeg_raw (unsigned char *jpeg_data, int len, int quality,
          for (y = 0; y < 8; y++) {
             row1[y] = &raw1[yc * width / 2];
             row2[y] = &raw2[yc * width / 2];
-            if (y%2) yc += numfields;
+            if ((ctype == CHROMA422) || (y%2))
+               yc += numfields;
          }
 
          jpeg_write_raw_data (&cinfo, scanarray,
