@@ -37,12 +37,15 @@
 
 #ifdef HAVE_LIBQUICKTIME
 #include <quicktime.h>
+#include <lqt.h>
 #endif
 
 extern int AVI_errno;
 
 static char video_format=' ';
 static int  internal_error=0;
+int libdv_pal_yv12 = -1;
+uint16_t reorder_16(uint16_t todo, int big_endian);
 
 #define ERROR_JPEG      1
 #define ERROR_MALLOC    2
@@ -71,8 +74,6 @@ static unsigned long jpeg_app1_offset    = 0;
 
 #define QUICKTIME_MJPG_TAG 0x6d6a7067  /* 'mjpg' */
 
-static int check_YUV420_input(lav_file_t *lav_fd);
-
 #ifdef HAVE_LIBDV
 static int check_DV2_input(lav_file_t *lav_fd);
 #endif
@@ -275,8 +276,9 @@ lav_file_t *lav_open_output_file(char *filename, char format,
                     int asize, int achans, long arate)
 {
    lav_file_t *lav_fd = (lav_file_t*) malloc(sizeof(lav_file_t));
+   char *extension, *tempfile;
 
-   if(lav_fd==0) { internal_error=ERROR_MALLOC; return 0; }
+   if (lav_fd == 0) { internal_error=ERROR_MALLOC; return 0; }
 
    /* Set lav_fd */
 
@@ -284,35 +286,47 @@ lav_file_t *lav_open_output_file(char *filename, char format,
    lav_fd->qt_fd       = 0;
    lav_fd->format      = format;
    /* Sanity check: do not create a quicktime file that is named with .avi */
-   if(rindex(filename, '.') != NULL)
+   extension = rindex(filename, '.');
+   if (extension != NULL)
    {
-      if((format == 'a' || format == 'A') && strcmp(rindex(filename, '.')+1, "avi")) {
-        internal_error = ERROR_FORMAT;
-        return 0;
-      }
-      if(format == 'q' && (strcmp(rindex(filename, '.')+1, "qt") 
-           && strcmp(rindex(filename, '.')+1, "mov") && strcmp(rindex(filename, '.')+1,"moov"))) {
-        internal_error = ERROR_FORMAT;
-        return 0;
-      }
-      if(format == 'j' && strcmp(rindex(filename, '.')+1, "jpg")
-           && strcmp(rindex(filename, '.')+1, "jpeg")) {
-        internal_error = ERROR_FORMAT;
-        return 0;
-      }
+      extension++;
+      switch(format)
+        {
+        case 'a':
+        case 'A':
+          if (strcasecmp(extension, "avi"))
+            {
+               internal_error = ERROR_FORMAT;
+               return 0;
+            }
+          break;
+        case 'q':
+          if (strcasecmp(extension,  "qt") &&
+              strcasecmp(extension, "mov") && strcasecmp(extension,"moov"))
+            {
+             internal_error = ERROR_FORMAT;
+             return 0;
+            }
+          break;
+        case 'j':
+          if (strcasecmp(extension, "jpg") && strcasecmp(extension,"jpeg"))
+            {
+              internal_error = ERROR_FORMAT;
+              return 0;
+            }
+          break;
+        }
    }
    lav_fd->interlacing = interlaced ? lav_query_polarity(format) :
                                       LAV_NOT_INTERLACED;
    lav_fd->has_audio   = (asize>0 && achans>0);
    lav_fd->bps         = (asize*achans+7)/8;
-   lav_fd->is_MJPG     = 1;
-   lav_fd->MJPG_chroma = CHROMAUNKNOWN;
+   lav_fd->chroma = CHROMAUNKNOWN;
 
    switch(format)
    {
       case 'a':
       case 'A':
-
          /* Open AVI output file */
 
          lav_fd->avi_fd = AVI_open_output_file(filename);
@@ -321,21 +335,22 @@ lav_file_t *lav_open_output_file(char *filename, char format,
          if (asize) AVI_set_audio(lav_fd->avi_fd, achans, arate, asize, WAVE_FORMAT_PCM);
          return lav_fd;
 
-      case 'j': {
-
+      case 'j':
         /* Open JPEG output file */
-        char tempfile[256];
-
+	tempfile = (char *)malloc(strlen(filename) + strlen(TMP_EXTENSION) + 1);
+	if (tempfile == NULL)
+	   {
+	   internal_error=ERROR_MALLOC;
+	   return(0);
+	   }
         strcpy(tempfile, filename);
         strcat(tempfile, TMP_EXTENSION);
-
         lav_fd->jpeg_filename = strdup(filename);
         lav_fd->jpeg_fd = open(tempfile, O_CREAT | O_TRUNC | O_WRONLY, 0644);
-
+	free(tempfile);
         return lav_fd;
-      }
-      case 'q':
 
+      case 'q':
 #ifdef HAVE_LIBQUICKTIME
          /* open quicktime output file */
 
@@ -348,11 +363,8 @@ lav_file_t *lav_open_output_file(char *filename, char format,
          if(!lav_fd->qt_fd) { free(lav_fd); return 0; }
          quicktime_set_video(lav_fd->qt_fd, 1, width, height, fps,
                              (interlaced ? QUICKTIME_MJPA : QUICKTIME_JPEG));
-
-         /* The sound system wants unsigned data (QUICKTIME_RAW) for 8 bit
-            and signed twos complement data (QUICKTIME_TWOS) for 16 bit! */
-         if (asize) quicktime_set_audio(lav_fd->qt_fd, achans, arate,
-                             asize, (asize==8) ? QUICKTIME_RAW : QUICKTIME_TWOS);
+         if (asize)
+	    quicktime_set_audio(lav_fd->qt_fd, achans, arate, asize, QUICKTIME_TWOS);
          return lav_fd;
 #else
 	 internal_error = ERROR_FORMAT;
@@ -364,26 +376,33 @@ lav_file_t *lav_open_output_file(char *filename, char format,
 }
 
 int lav_close(lav_file_t *lav_file)
-{
+   {
    int res;
+   char *tempfile;
 
    video_format = lav_file->format; internal_error = 0; /* for error messages */
 
-   switch(lav_file->format)
-   {
+   switch (lav_file->format)
+      {
       case 'a':
       case 'A':
          res = AVI_close( lav_file->avi_fd );
          break;
-      case 'j': {
-         char tempfile[256];
+      case 'j':
+	 tempfile = (char *)malloc(strlen(lav_file->jpeg_filename) + 
+				   strlen(TMP_EXTENSION) + 1);
+	 if (tempfile == NULL)
+	    {
+	    res = -1;
+	    break;
+	    }
          strcpy(tempfile, lav_file->jpeg_filename);
          strcat(tempfile, TMP_EXTENSION);
          res = close(lav_file->jpeg_fd);
          rename(tempfile, lav_file->jpeg_filename);
+	 free(tempfile);
          free(lav_file->jpeg_filename);
          break;
-      }
 #ifdef HAVE_LIBQUICKTIME
       case 'q':
          res = quicktime_close( lav_file->qt_fd );
@@ -391,12 +410,10 @@ int lav_close(lav_file_t *lav_file)
 #endif
       default:
          res = -1;
-   }
-
+      }
    free(lav_file);
-
    return res;
-}
+   }
 
 int lav_write_frame(lav_file_t *lav_file, uint8_t *buff, long size, long count)
 {
@@ -524,9 +541,14 @@ int lav_write_frame(lav_file_t *lav_file, uint8_t *buff, long size, long count)
 int lav_write_audio(lav_file_t *lav_file, uint8_t *buff, long samps)
 {
    int res;
-#ifdef	HAVE_LIBQUICKTIME
-   int	n, nb;
-   uint8_t *hbuff;
+#ifdef HAVE_LIBQUICKTIME
+   int i, j;
+   int16_t *qt_audio = (int16_t *)buff, **qt_audion;
+   int channels = lav_audio_channels(lav_file);
+
+   qt_audion = malloc(channels * sizeof (int16_t **));
+   for (i = 0; i < channels; i++)
+	qt_audion[i] = (int16_t *)malloc(samps * lav_file->bps);
 #endif
 
    video_format = lav_file->format; internal_error = 0; /* for error messages */
@@ -539,19 +561,17 @@ int lav_write_audio(lav_file_t *lav_file, uint8_t *buff, long samps)
          break;
 #ifdef HAVE_LIBQUICKTIME
       case 'q':
-         if(lav_audio_bits(lav_file)==16)
-         {
-            nb = samps*2*lav_audio_channels(lav_file);
-            hbuff = (uint8_t *) malloc(nb);
-            if(!hbuff) { internal_error=ERROR_MALLOC; return -1; }
-            for(n=0;n<nb;n+=2)
-            { hbuff[n] = buff[n+1]; hbuff[n+1] = buff[n]; }
-            res = quicktime_write_audio( lav_file->qt_fd, (char*)hbuff, samps, 0 );
-            free(hbuff);
-         }
-         else
-            res = quicktime_write_audio( lav_file->qt_fd, (char*)buff, samps, 0 );
-         break;
+	/* Deinterleave the audio into the two channels. */
+	for (i = 0; i < samps; i++)
+	    {
+	    for (j = 0; j < channels; j++)
+		qt_audion[j][i] = qt_audio[(channels*i) + j];
+	    }
+	res = lqt_encode_audio_track(lav_file->qt_fd, qt_audion, NULL,samps,0);
+	for (j = 0; j < channels; j++)
+	    free(qt_audion[j]);
+	free(qt_audion);
+        break;
 #endif
       default:
          res = -1;
@@ -560,8 +580,6 @@ int lav_write_audio(lav_file_t *lav_file, uint8_t *buff, long samps)
    return res;
 }
 
-
-
 long lav_video_frames(lav_file_t *lav_file)
 {
    video_format = lav_file->format; internal_error = 0; /* for error messages */
@@ -638,17 +656,11 @@ void lav_video_sampleaspect(lav_file_t *lav_file, int *sar_w, int *sar_h)
   return;
 }
 
-int lav_video_is_MJPG(lav_file_t *lav_file)
+int lav_video_chroma(lav_file_t *lav_file)
 {
-   return lav_file->is_MJPG;
+	return lav_file->chroma;
 }
 
-int lav_video_MJPG_chroma(lav_file_t *lav_file)
-{
-	return lav_file->MJPG_chroma;
-}
-
-
 const char *lav_video_compressor(lav_file_t *lav_file)
 {
    video_format = lav_file->format; internal_error = 0; /* for error messages */
@@ -797,7 +809,6 @@ int lav_read_frame(lav_file_t *lav_file, uint8_t *vidbuf)
    return -1;
 }
 
-
 int lav_set_audio_position(lav_file_t *lav_file, long sample)
 {
    if(!lav_file->has_audio) return 0;
@@ -809,7 +820,7 @@ int lav_set_audio_position(lav_file_t *lav_file, long sample)
          return AVI_set_audio_position(lav_file->avi_fd,sample*lav_file->bps);
 #ifdef HAVE_LIBQUICKTIME
       case 'q':
-         return quicktime_set_audio_position(lav_file->qt_fd,sample,0);
+         quicktime_set_audio_position(lav_file->qt_fd,sample,0);
 #endif
    }
    return -1;
@@ -818,13 +829,21 @@ int lav_set_audio_position(lav_file_t *lav_file, long sample)
 long lav_read_audio(lav_file_t *lav_file, uint8_t *audbuf, long samps)
 {
 #ifdef	HAVE_LIBQUICKTIME
-   long res, n, t;
+   int64_t last_pos, start_pos;
+   int res, i, j;
+   int16_t *qt_audio = (int16_t *)audbuf, **qt_audion;
+   int channels = lav_audio_channels(lav_file);
+   uint8_t b0, b1;
+
+   qt_audion = malloc(channels * sizeof (int16_t **));
+   for (i = 0; i < channels; i++)
+	qt_audion[i] = (int16_t *)malloc(samps * lav_file->bps);
 #endif
 
    if(!lav_file->has_audio)
    {
       internal_error = ERROR_NOAUDIO;
-      return -1;
+      return(-1);
    }
    video_format = lav_file->format; internal_error = 0; /* for error messages */
    switch(lav_file->format)
@@ -834,28 +853,43 @@ long lav_read_audio(lav_file_t *lav_file, uint8_t *audbuf, long samps)
          return AVI_read_audio(lav_file->avi_fd,audbuf,samps*lav_file->bps)/lav_file->bps;
 #ifdef HAVE_LIBQUICKTIME
       case 'q':
-         res = quicktime_read_audio(lav_file->qt_fd,(char*)audbuf,samps,0)/lav_file->bps;
-         if(res<=0) return res;
-         if(lav_audio_bits(lav_file)==16)
-         {
-            for(n=0;n<res*2*lav_audio_channels(lav_file);n+=2)
-            {
-               t = audbuf[n];
-               audbuf[n] = audbuf[n+1];
-               audbuf[n+1] = t;
+	start_pos = quicktime_audio_position(lav_file->qt_fd, 0);
+	lqt_decode_audio_track(lav_file->qt_fd, qt_audion, NULL, samps, 0);
+	last_pos = lqt_last_audio_position(lav_file->qt_fd, 0);
+	res = last_pos - start_pos;
+	if (res <= 0)
+	   goto out;
+	/* Interleave the channels of audio into the one buffer provided */
+	for (i =0; i < res; i++)
+	    {
+	    for (j = 0; j < channels; j++)
+		qt_audio[(channels*i) + j] = qt_audion[j][i];
+	    }
+
+        if (lav_detect_endian())
+           {
+           i= 0;
+           while (i < (2*res) )
+                 {
+                 b0 = 0;
+                 b1 = 0; 
+                 b0 = (qt_audio[i] & 0x00FF);
+                 b1 =  (qt_audio[i] & 0xFF00) >> 8;
+                 qt_audio[i] = (b0 <<8) + b1;
+                 i = i +1;
+                 } 
             }
-         }
-         return res;
+out:
+	for (j = 0; j < channels; j++)
+            qt_audio[(channels*i) + j] = qt_audion[j][i];
+
+	 free(qt_audion);
+         return(res);
 #endif
    }
    return -1;
 }
 
-int lav_filetype(lav_file_t *lav_file)
-{
-   return lav_file->format;
-}
-
 lav_file_t *lav_open_input_file(char *filename)
 {
    int n;
@@ -878,12 +912,11 @@ lav_file_t *lav_open_input_file(char *filename)
    lav_fd->qt_fd       = 0;
    lav_fd->format      = 0;
    lav_fd->interlacing = LAV_INTER_UNKNOWN;
-   lav_fd->sar_w       = 0; /* (0,0) == unknown */
-   lav_fd->sar_h       = 0; 
+   lav_fd->sar_w       = 1; /* unknown - assume square pixels */
+   lav_fd->sar_h       = 1; 
    lav_fd->has_audio   = 0;
    lav_fd->bps         = 0;
-   lav_fd->is_MJPG     = 0;
-   lav_fd->MJPG_chroma = CHROMAUNKNOWN;
+   lav_fd->chroma = CHROMAUNKNOWN;
 
    /* open video file, try AVI first */
 
@@ -936,21 +969,13 @@ lav_file_t *lav_open_input_file(char *filename)
 	    }
 	  /* Check for audio tracks */
 	  lav_fd->has_audio = 0;
-	  if(quicktime_audio_tracks(lav_fd->qt_fd))
-	    {
-	      audio_comp = quicktime_audio_compressor(lav_fd->qt_fd,0);
-	      /* in order to be able to play the audio correctly,
-		 size must either be 8 bits and compressor "raw "
-		 or 16 bits and compressor "twos" */
-	      if( ( quicktime_audio_bits(lav_fd->qt_fd,0)==8 &&
-		    strncasecmp(audio_comp,QUICKTIME_RAW,4)==0 ) ||
-		  ( quicktime_audio_bits(lav_fd->qt_fd,0)==16 &&
-		    strncasecmp(audio_comp,QUICKTIME_TWOS,4)==0 ) )
-		{
-		  lav_fd->has_audio = 1;
-		}
-	    }
-	}
+	  if (quicktime_audio_tracks(lav_fd->qt_fd))
+	     {
+	     audio_comp = quicktime_audio_compressor(lav_fd->qt_fd,0);
+	     if (strncasecmp(audio_comp, QUICKTIME_TWOS,4)==0)
+		lav_fd->has_audio = 1;
+	      }
+	 }
 #endif
    }
    else
@@ -967,6 +992,8 @@ lav_file_t *lav_open_input_file(char *filename)
 
    /* Check compressor, no further action if not Motion JPEG */
 
+   ierr  = 0;
+
    if(strncasecmp(video_comp,"mjpg",4)!=0 &&
       strncasecmp(video_comp,"mjpa",4)!=0 &&
       strncasecmp(video_comp,"jpeg",4)!=0 )
@@ -978,43 +1005,22 @@ lav_file_t *lav_open_input_file(char *filename)
 #endif
       || strncasecmp(video_comp,"dv",2)==0) {
        ierr = check_DV2_input(lav_fd);
-#ifdef LIBDV_PAL_YV12
-       lav_fd->MJPG_chroma = CHROMA420;
-#else
-       lav_fd->MJPG_chroma = CHROMA422;
-#endif
        if (ierr) goto ERREXIT;
        /* DV is always interlaced, bottom first */
        lav_fd->interlacing = LAV_INTER_BOTTOM_FIRST; 
    }
 #endif /* HAVE_LIBDV */
 
-   if(strncasecmp(video_comp,"yuv",3)==0
-#ifdef HAVE_LIBQUICKTIME
-/*    || strncasecmp(video_comp,QUICKTIME_YUV4,4)==0 */
-      || strncasecmp(video_comp,QUICKTIME_YUV420,4)==0
-#endif
-      ) {
-       ierr = check_YUV420_input(lav_fd);
-#ifdef HAVE_LIBQUICKTIME
-       /* check for YUV format if quicktime file */
-       if (strncasecmp(video_comp,QUICKTIME_YUV420,4)==0)
-           lav_fd->MJPG_chroma = CHROMA420;
-       else if (strncasecmp(video_comp,QUICKTIME_YUV4,4)==0)
-           lav_fd->MJPG_chroma = CHROMA422;
-#else
-   lav_fd->MJPG_chroma = CHROMA420;
-#endif
-       if (ierr) goto ERREXIT;
+   if (strncasecmp(video_comp,"yuv",3)==0 || strncasecmp(video_comp, "yv12",4)==0)
+      {
+      lav_fd->chroma = CHROMA420;
+      if (ierr) goto ERREXIT;
+      }
+   return lav_fd;
    }
-      return lav_fd;
-   }
-
-   lav_fd->is_MJPG = 1;
 
    /* Make some checks on the video source, we read the first frame for that */
 
-   ierr  = 0;
    frame = NULL;
    if ( lav_set_video_position(lav_fd,0) ) goto ERREXIT;
    if ( (len = lav_frame_size(lav_fd,0)) <=0 ) goto ERREXIT;
@@ -1064,15 +1070,15 @@ lav_file_t *lav_open_input_file(char *filename)
 	  {
 		 if( vf[0] == vf[1] && vf[0] == vf[2] )
 		 {
-			 lav_fd->MJPG_chroma = CHROMA422;
+			 lav_fd->chroma = CHROMA422;
 		 }
 		 else if( vf[0] == 2*vf[1] && vf[0] == 2*vf[2] )
-			 lav_fd->MJPG_chroma = CHROMA420;
+			 lav_fd->chroma = CHROMA420;
 		 else		
-			 lav_fd->MJPG_chroma = CHROMAUNKNOWN;
+			 lav_fd->chroma = CHROMAUNKNOWN;
 	  }
 	  else
-		  lav_fd->MJPG_chroma = CHROMAUNKNOWN;
+		  lav_fd->chroma = CHROMAUNKNOWN;
    }
 
    /* Check if video is interlaced */
@@ -1200,95 +1206,153 @@ const char *lav_strerror(void)
    }
 }
 
-
-
 #ifdef HAVE_LIBDV
 static int check_DV2_input(lav_file_t *lav_fd)
-{
-   int ierr = 0;
-   double len = 0;
-   unsigned char *frame = NULL;
-
-   lav_fd->is_MJPG = 0;
+	{
+	int ierr = 0;
+	double len = 0;
+	unsigned char *frame = NULL;
+	dv_decoder_t *decoder = NULL;
+	uint8_t *output[3] = { NULL, NULL, NULL };
+	uint16_t pitches[3];
 
    /* Make some checks on the video source, we read the first frame for that */
 
-   if ( lav_set_video_position(lav_fd,0) ) goto ERREXIT;
-   if ( (len = lav_frame_size(lav_fd,0)) <=0 ) goto ERREXIT;
-   if ( (frame = (unsigned char*) malloc(len)) == 0 ) { ierr=ERROR_MALLOC; goto ERREXIT; }
+	if	(lav_set_video_position(lav_fd,0))
+		goto ERREXIT;
+	if	((len = lav_frame_size(lav_fd,0)) <=0)
+		goto ERREXIT;
+	if	((frame = (unsigned char*) malloc(len)) == 0)
+		{
+		ierr=ERROR_MALLOC;
+		goto ERREXIT;
+		}
 
-   if ( lav_read_frame(lav_fd,frame) <= 0 ) goto ERREXIT;
-   {
-     dv_decoder_t *decoder = dv_decoder_new(0,0,0);
-     dv_parse_header(decoder, frame);
+	if	(lav_read_frame(lav_fd, frame) <= 0)
+		goto ERREXIT;
 
-     switch (decoder->system) {
-     case e_dv_system_525_60:
-       if (dv_format_wide(decoder)) {
-	 lav_fd->sar_w = 40;
-	 lav_fd->sar_h = 33;
-       } else {
-	 lav_fd->sar_w = 10;
-	 lav_fd->sar_h = 11;
-       } 
-       break;
-     case e_dv_system_625_50:
-       if (dv_format_wide(decoder)) {
-	 lav_fd->sar_w = 118;
-	 lav_fd->sar_h = 81;
-       } else {
-	 lav_fd->sar_w = 59;
-	 lav_fd->sar_h = 54;
-       } 
-       break;
-     default:
-       lav_fd->sar_w = 0; /* ??? -> unknown */
-       lav_fd->sar_h = 0;
-       break;
-     }
-     dv_decoder_free(decoder);
-   }
+	decoder = dv_decoder_new(0,0,0);
+	dv_parse_header(decoder, frame);
 
-   /* reset video position to 0 */
-   if ( lav_set_video_position(lav_fd,0) ) goto ERREXIT;
-   return 0;
+	switch	(decoder->system)
+		{
+		case	e_dv_system_525_60:
+			if	(dv_format_wide(decoder))
+				{
+				lav_fd->sar_w = 40;
+				lav_fd->sar_h = 33;
+				}
+			else	{
+	 			lav_fd->sar_w = 10;
+	 			lav_fd->sar_h = 11;
+       				} 
+       			break;
+		case	e_dv_system_625_50:
+			if	(dv_format_wide(decoder))
+				{
+				lav_fd->sar_w = 118;
+				lav_fd->sar_h = 81;
+			} else {
+				lav_fd->sar_w = 59;
+				lav_fd->sar_h = 54;
+				} 
+			break;
+		default:
+			lav_fd->sar_w = 0; /* ??? -> unknown */
+			lav_fd->sar_h = 0;
+			break;
+		}
+
+	switch(decoder->sampling) {
+	case e_dv_sample_420:
+	  /* libdv decodes PAL DV directly as planar YUV 420
+	   * (YV12 or 4CC 0x32315659) if configured with the flag
+	   * --with-pal-yuv=YV12 which is not (!) the default
+	   */
+	  if (libdv_pal_yv12 < 0 ) {
+	    /* PAL YV12 not already detected.
+	     * Setting decoding options for 4:2:2, because libdv pitches
+	     * is *int* for YUY2 and *uint16_t* for YV12 (no comment...)
+	     * In YV12 mode, even if pitches is 0, the first line is filled.
+	     */
+
+	    pitches[0] = decoder->width * 2;
+	    pitches[1] = 0;
+	    pitches[2] = 0;
+
+	    /* Hacking libdv: set invalid values for chroma. If libdv
+	     * was compiled with PAL YV12, the values are overwritten,
+	     * otherwise (4:2:2 output) only dv_frame[0] is filled and
+	     * chroma planes remain untouched.
+	     */
+	    output[0] = (uint8_t *)malloc(decoder->width * decoder->height * 2);
+	    output[1] = (uint8_t *)malloc(decoder->width * decoder->height / 2);
+	    if (!output[0] || !output[1])
+	    {
+	      ierr=ERROR_MALLOC;
+	      goto ERREXIT;
+	    } else {
+	      output[2] = output[1] + (decoder->width * decoder->height / 4 );
+	      output[1][0] = 0;
+	      output[1][1] = 255;
+	      output[2][0] = 255;
+	      output[2][1] = 0;
+	      dv_decode_full_frame(decoder, frame, e_dv_color_yuv,
+				   output, (int *)pitches);
+	      if (output[1][0]==0   && output[1][1]==255 &&
+		  output[2][0]==255 && output[2][1]==0) {
+		libdv_pal_yv12 = 0;
+		lav_fd->chroma = CHROMA422;
+		mjpeg_info("Detected libdv PAL output YUY2 (4:2:2)");
+	      } else {
+		libdv_pal_yv12 = 1;
+		lav_fd->chroma = CHROMA420;
+		mjpeg_info("Detected libdv PAL output YV12 (4:2:0)");
+	      }
+	      free(output[0]);
+	      free(output[1]);
+	    }
+	  } else {
+	    if(libdv_pal_yv12 == 0)
+	      lav_fd->chroma = CHROMA422;
+	    else
+	      lav_fd->chroma = CHROMA420;
+	  }
+	  break;
+	case e_dv_sample_411:
+	case e_dv_sample_422:
+	   /* libdv decodes NTSC DV (native 411) and by default also PAL
+	    * DV (native 420) as packed YUV 422 (YUY2 or 4CC 0x32595559)
+	    * where the U and V information is repeated.  This can be
+	    * transformed to planar 420 (YV12 or 4CC 0x32315659).
+	    * For NTSC DV this transformation is lossy.
+	    */
+	   lav_fd->chroma = CHROMA422;
+	   break;
+	case e_dv_sample_none:
+	   /* FIXME */
+	   break;
+	}
+	dv_decoder_free(decoder);
+	lav_set_video_position(lav_fd,0);
+	return 0;
 
 ERREXIT:
-   lav_close(lav_fd);
-   if(frame) free(frame);
-   if (ierr) internal_error = ierr;
-   return 1;
-}
+	lav_close(lav_fd);
+	if	(frame)
+		free(frame);
+	if	(decoder)
+		dv_decoder_free(decoder);
+	if	(output[0])
+		free(output[0]);
+	if	(output[1])
+		free(output[1]);
+	if	(ierr)
+		internal_error = ierr;
+	return 1;
+	}
 #endif
 
-
-
-static int check_YUV420_input(lav_file_t *lav_fd)
-{
-   int ierr = 0;
-   double len = 0;
-   unsigned char *frame = NULL;
-
-   lav_fd->is_MJPG = 0;
-
-   /* Make some checks on the video source, we read the first frame for that */
-
-   if ( lav_set_video_position(lav_fd,0) ) goto ERREXIT;
-   if ( (len = lav_frame_size(lav_fd,0)) <=0 ) goto ERREXIT;
-   if ( (frame = (unsigned char*) malloc(len)) == 0 ) { ierr=ERROR_MALLOC; goto ERREXIT; }
-
-   if ( lav_read_frame(lav_fd,frame) <= 0 ) goto ERREXIT;
-   /* reset video position to 0 */
-   if ( lav_set_video_position(lav_fd,0) ) goto ERREXIT;
-   return 0;
-
-ERREXIT:
-   lav_close(lav_fd);
-   if(frame) free(frame);
-   if (ierr) internal_error = ierr;
-   return 1;
-}
-
 int lav_fileno(lav_file_t *lav_file)
 {
    int res;
@@ -1313,16 +1377,42 @@ int lav_fileno(lav_file_t *lav_file)
    return res;
 }
 
+/* We need this to reorder the 32 bit values for big endian systems */
+uint32_t reorder_32(uint32_t todo, int big_endian)
+{
+  unsigned char b0, b1, b2, b3;
+  unsigned long reversed; 
 
+  if( big_endian )
+  {
+    b0 = (todo & 0x000000FF);
+    b1 = (todo & 0x0000FF00) >> 8;
+    b2 = (todo & 0x00FF0000) >> 16;
+    b3 = (todo & 0xFF000000) >> 24;
 
+    reversed = (b0 << 24) + (b1 << 16) + (b2 << 8) +b3;
+    return reversed;
+  }
+  else
+  {
+    return todo;
+  }
 
+}
 
+int lav_detect_endian (void)
+{
+    unsigned int fred;
+    char     *pfred;
 
+  fred = 2 | (1 << (sizeof(int)*8-8));
+  pfred = (char *)&fred;
 
-
-
-
-
-
-
+  if  (*pfred == 1)
+      return 1;
+  else if(*pfred == 2)
+      return 0;
+  else
+      return -1;
+}
 
