@@ -5,7 +5,7 @@
  *               performs 4:2:0 chroma subsampling.
  *
  *
- *  Copyright (C) 2001 Matthew J. Marjanovic <maddog@mir.com>
+ *  Copyright (C) 2004 Matthew J. Marjanovic <maddog@mir.com>
  *
  *
  *  This program is free software; you can redistribute it and/or
@@ -42,6 +42,9 @@
 # define O_BINARY 0
 #endif
 
+#define DEFAULT_CHROMA_MODE Y4M_CHROMA_444
+
+
 /* command-line parameters */
 typedef struct _cl_info {
   y4m_ratio_t aspect;    
@@ -54,6 +57,7 @@ typedef struct _cl_info {
   int ss_mode;
   int verbosity;
   int fdin;
+  int bgr;
 } cl_info_t;
 
 
@@ -68,7 +72,6 @@ typedef struct _ppm_info {
 static
 void usage(const char *progname)
 {
-  int i;
   fprintf(stdout, "\n");
   fprintf(stdout, "usage:  %s [options] [ppm-file]\n", progname);
   fprintf(stdout, "\n");
@@ -90,12 +93,22 @@ void usage(const char *progname)
   fprintf(stdout, "             b = bottom-field-first\n");
   fprintf(stdout, "  -L       treat PPM images as 2-field interleaved\n");
   fprintf(stdout, "  -r       repeat last input frame\n");
-  fprintf(stdout, "  -S mode  chroma subsampling mode [%s]\n",
-	  ssm_id[SSM_420_JPEG]);
-  for (i = 1; i < SSM_COUNT; i++)
-  fprintf(stdout, "            '%s' -> %s\n", ssm_id[i], ssm_description[i]);
+  {
+    int m;
+    const char *keyword;
+
+    fprintf(stdout, "  -S mode  chroma subsampling mode [%s]\n",
+	    y4m_chroma_keyword(DEFAULT_CHROMA_MODE));
+    for (m = 0;
+	 (keyword = y4m_chroma_keyword(m)) != NULL;
+	 m++)
+      if (chroma_sub_implemented(m))
+	fprintf(stdout, "            '%s' -> %s\n",
+		keyword, y4m_chroma_description(m));
+  }
   /*  fprintf(stdout, "  -R type  subsampling filter type\n");*/
   fprintf(stdout, "  -v n     verbosity (0,1,2) [1]\n");
+  fprintf(stdout, "  -B       PPM image is packed with BGR pixels [RGB]\n");
 }
 
 
@@ -103,7 +116,6 @@ void usage(const char *progname)
 static
 void parse_args(cl_info_t *cl, int argc, char **argv)
 {
-  int i;
   int c;
 
   cl->offset = 0;
@@ -113,11 +125,12 @@ void parse_args(cl_info_t *cl, int argc, char **argv)
   cl->framerate = y4m_fps_NTSC;
   cl->interleave = 0;
   cl->repeatlast = 0;
-  cl->ss_mode = SSM_420_JPEG;
+  cl->ss_mode = DEFAULT_CHROMA_MODE;
   cl->verbosity = 1;
+  cl->bgr = 0;
   cl->fdin = 0; /* default to stdin */
 
-  while ((c = getopt(argc, argv, "A:F:I:Lo:n:rS:v:h")) != -1) {
+  while ((c = getopt(argc, argv, "BA:F:I:Lo:n:rS:v:h")) != -1) {
     switch (c) {
     case 'A':
       if (y4m_parse_ratio(&(cl->aspect), optarg) != Y4M_OK) {
@@ -145,6 +158,9 @@ void parse_args(cl_info_t *cl, int argc, char **argv)
     case 'L':
       cl->interleave = 1;
       break;
+    case 'B':
+      cl->bgr = 1;
+      break;
     case 'o':
       if ((cl->offset = atoi(optarg)) < 0)
 	mjpeg_error_exit1("Offset must be >= 0:  '%s'", optarg);
@@ -157,14 +173,13 @@ void parse_args(cl_info_t *cl, int argc, char **argv)
       cl->repeatlast = 1;
       break;
     case 'S':
-      for (i = 0; i < SSM_COUNT; i++) {
-	if (!(strcmp(optarg, ssm_id[i]))) break;
-      }
-      if (i < SSM_COUNT)
-	cl->ss_mode = i;
-      else {
+      cl->ss_mode = y4m_chroma_parse_keyword(optarg);
+      if (cl->ss_mode == Y4M_UNKNOWN) {
 	mjpeg_error("Unknown subsampling mode option:  %s", optarg);
 	goto ERROR_EXIT;
+      } else if (!chroma_sub_implemented(cl->ss_mode)) {
+	mjpeg_error("Unsupported subsampling mode option:  %s", optarg);
+	goto ERROR_EXIT;
       }
       break;
     case 'v':
@@ -198,6 +213,8 @@ void parse_args(cl_info_t *cl, int argc, char **argv)
 	     cl->framerate.n, cl->framerate.d);
   mjpeg_info("    pixel aspect ratio:  %d:%d",
 	     cl->aspect.n, cl->aspect.d);
+  mjpeg_info("         pixel packing:  %s",
+	     cl->bgr?"BGR":"RGB");
   mjpeg_info("             interlace:  %s%s",
 	     mpeg_interlace_code_definition(cl->interlace),
 	     (cl->interlace == Y4M_ILACE_NONE) ? "" :
@@ -213,7 +230,8 @@ void parse_args(cl_info_t *cl, int argc, char **argv)
 	       cl->framecount,
 	       (cl->repeatlast) ? ", repeat last frame until done" :
 	       ", or until input exhausted");
-  mjpeg_info("    chroma subsampling:  %s", ssm_description[cl->ss_mode]);
+  mjpeg_info("    chroma subsampling:  %s",
+	     y4m_chroma_description(cl->ss_mode));
 
   /* DONE! */
   return;
@@ -307,7 +325,7 @@ void read_ppm_into_two_buffers(int fd,
 			       uint8_t *buffers[], 
 			       uint8_t *buffers2[], 
 			       uint8_t *rowbuffer,
-			       int width, int height)
+			       int width, int height, int bgr)
 {
   int x, y;
   uint8_t *pixels;
@@ -324,18 +342,34 @@ void read_ppm_into_two_buffers(int fd,
     pixels = rowbuffer;
     if (y4m_read(fd, pixels, width * 3))
       mjpeg_error_exit1("read error A  y=%d", y);
-    for (x = 0; x < width; x++) {
-      *(R++) = *(pixels++);
-      *(G++) = *(pixels++);
-      *(B++) = *(pixels++);
+    if (bgr) {
+      for (x = 0; x < width; x++) {
+	*(B++) = *(pixels++);
+	*(G++) = *(pixels++);
+	*(R++) = *(pixels++);
+      }
+    } else {
+      for (x = 0; x < width; x++) {
+	*(R++) = *(pixels++);
+	*(G++) = *(pixels++);
+	*(B++) = *(pixels++);
+      }
     }
     pixels = rowbuffer;
     if (y4m_read(fd, pixels, width * 3))
       mjpeg_error_exit1("read error B  y=%d", y);
-    for (x = 0; x < width; x++) {
-      *(R2++) = *(pixels++);
-      *(G2++) = *(pixels++);
-      *(B2++) = *(pixels++);
+    if (bgr) {
+      for (x = 0; x < width; x++) {
+	*(B2++) = *(pixels++);
+	*(G2++) = *(pixels++);
+	*(R2++) = *(pixels++);
+      }
+    } else {
+      for (x = 0; x < width; x++) {
+	*(R2++) = *(pixels++);
+	*(G2++) = *(pixels++);
+	*(B2++) = *(pixels++);
+      }
     }
   }
 }
@@ -346,7 +380,7 @@ static
 void read_ppm_into_one_buffer(int fd,
 			      uint8_t *buffers[],
 			      uint8_t *rowbuffer,
-			      int width, int height) 
+			      int width, int height, int bgr) 
 {
   int x, y;
   uint8_t *pixels;
@@ -357,10 +391,18 @@ void read_ppm_into_one_buffer(int fd,
   for (y = 0; y < height; y++) {
     pixels = rowbuffer;
     y4m_read(fd, pixels, width * 3);
-    for (x = 0; x < width; x++) {
-      *(R++) = *(pixels++);
-      *(G++) = *(pixels++);
-      *(B++) = *(pixels++);
+    if (bgr) {
+      for (x = 0; x < width; x++) {
+	*(B++) = *(pixels++);
+	*(G++) = *(pixels++);
+	*(R++) = *(pixels++);
+      }
+    } else {
+      for (x = 0; x < width; x++) {
+	*(R++) = *(pixels++);
+	*(G++) = *(pixels++);
+	*(B++) = *(pixels++);
+      }
     }
   }
 }
@@ -378,7 +420,7 @@ void read_ppm_into_one_buffer(int fd,
 static
 int read_ppm_frame(int fd, ppm_info_t *ppm,
 		   uint8_t *buffers[], uint8_t *buffers2[],
-		   int ilace, int ileave)
+		   int ilace, int ileave, int bgr)
 {
   int width, height;
   static uint8_t *rowbuffer = NULL;
@@ -414,17 +456,17 @@ int read_ppm_frame(int fd, ppm_info_t *ppm,
     if (ilace == Y4M_ILACE_TOP_FIRST) {
       /* 1st buff arg == top field == temporally first == "buffers" */
       read_ppm_into_two_buffers(fd, buffers, buffers2,
-				rowbuffer, width, height);
+				rowbuffer, width, height, bgr);
     } else {
       /* bottom-field-first */
       /* 1st buff art == top field == temporally second == "buffers2" */
       read_ppm_into_two_buffers(fd, buffers2, buffers,
-				rowbuffer, width, height);
+				rowbuffer, width, height, bgr);
     }      
   } else if ((ilace == Y4M_ILACE_NONE) || (!ileave)) {
     /* Not Interlaced, or Not Interleaved:
        --> read image into first buffer... */
-    read_ppm_into_one_buffer(fd, buffers, rowbuffer, width, height);
+    read_ppm_into_one_buffer(fd, buffers, rowbuffer, width, height, bgr);
     if ((ilace != Y4M_ILACE_NONE) && (!ileave)) {
       /* ...Actually Interlaced:
 	 --> read the second image/field into second buffer */
@@ -437,7 +479,7 @@ int read_ppm_frame(int fd, ppm_info_t *ppm,
       if ( (ppm->width != width) ||
 	   (ppm->height != height) )
 	mjpeg_error_exit1("One of these frames is not like the others!");
-      read_ppm_into_one_buffer(fd, buffers2, rowbuffer, width, height);
+      read_ppm_into_one_buffer(fd, buffers2, rowbuffer, width, height, bgr);
     }
   }
   return 0;
@@ -450,16 +492,20 @@ void setup_output_stream(int fdout, cl_info_t *cl,
 			 y4m_stream_info_t *sinfo, ppm_info_t *ppm,
 			 int *field_height) 
 {
-  if ((ppm->width % 2) != 0) {
-    mjpeg_error("PPM width (%d) is not a multiple of 2!", ppm->width);
-    exit(1);
+  int err;
+  int x_alignment = y4m_chroma_ss_x_ratio(cl->ss_mode).d;
+  int y_alignment = y4m_chroma_ss_y_ratio(cl->ss_mode).d;
+    
+  if (cl->interlace != Y4M_ILACE_NONE) 
+    y_alignment *= 2;
+
+  if ((ppm->width % x_alignment) != 0) {
+    mjpeg_error_exit1("PPM width (%d) is not a multiple of %d!",
+		      ppm->width, x_alignment);
   }
-  if (((ppm->height % 2) != 0) && (cl->interlace == Y4M_ILACE_NONE)) {
-    mjpeg_error("PPM height (%d) is not a multiple of 2!", ppm->height);
-    exit(1);
-  } else if (((ppm->height % 4) != 0) && (cl->interlace != Y4M_ILACE_NONE)) {
-    mjpeg_error("PPM height (%d) is not a multiple of 4!", ppm->height);
-    exit(1);
+  if ((ppm->height % y_alignment) != 0) {
+    mjpeg_error_exit1("PPM height (%d) is not a multiple of %d!",
+		      ppm->height, y_alignment);
   }
 
   y4m_si_set_width(sinfo, ppm->width);
@@ -476,8 +522,10 @@ void setup_output_stream(int fdout, cl_info_t *cl,
   y4m_si_set_sampleaspect(sinfo, cl->aspect);
   y4m_si_set_interlace(sinfo, cl->interlace);
   y4m_si_set_framerate(sinfo, cl->framerate);
+  y4m_si_set_chroma(sinfo, cl->ss_mode);
 
-  y4m_write_stream_header(fdout, sinfo);
+  if ((err = y4m_write_stream_header(fdout, sinfo)) != Y4M_OK)
+    mjpeg_error_exit1("Write header failed:  %s", y4m_strerr(err));
 
   mjpeg_info("Output Stream parameters:");
   y4m_log_stream_info(LOG_INFO, "  ", sinfo);
@@ -491,14 +539,15 @@ int main(int argc, char **argv)
   cl_info_t cl;
   y4m_stream_info_t sinfo;
   y4m_frame_info_t finfo;
-  uint8_t *buffers[3];  /* R'G'B' or Y'CbCr */
-  uint8_t *buffers2[3]; /* R'G'B' or Y'CbCr */
+  uint8_t *buffers[Y4M_MAX_NUM_PLANES];  /* R'G'B' or Y'CbCr */
+  uint8_t *buffers2[Y4M_MAX_NUM_PLANES]; /* R'G'B' or Y'CbCr */
   ppm_info_t ppm;
   int field_height;
 
   int fdout = 1;
   int err, i, count, repeating_last;
 
+  y4m_accept_extensions(1);
   y4m_init_stream_info(&sinfo);
   y4m_init_frame_info(&finfo);
 
@@ -513,7 +562,7 @@ int main(int argc, char **argv)
 
   /* Read first PPM frame/field-pair, to get dimensions */
   if (read_ppm_frame(cl.fdin, &ppm, buffers, buffers2, 
-		     cl.interlace, cl.interleave))
+		     cl.interlace, cl.interleave, cl.bgr))
     mjpeg_error_exit1("Failed to read first frame.");
 
   /* Setup streaminfo and write output header */
@@ -530,7 +579,7 @@ int main(int argc, char **argv)
     /* ...but skip reading very first frame, already read prior to loop */
     if (count > 0) {
       err = read_ppm_frame(cl.fdin, &ppm, buffers, buffers2, 
-			   cl.interlace, cl.interleave);
+			   cl.interlace, cl.interleave, cl.bgr);
       if (err == 1) {
 	/* clean input EOF */
 	if (cl.repeatlast) {
@@ -564,13 +613,18 @@ int main(int argc, char **argv)
     if (count >= cl.offset) {
       switch (cl.interlace) {
       case Y4M_ILACE_NONE:
-	y4m_write_frame(fdout, &sinfo, &finfo, buffers);
+	if ((err = y4m_write_frame(fdout, &sinfo, &finfo, buffers)) != Y4M_OK)
+	  mjpeg_error_exit1("Write frame failed: %s", y4m_strerr(err));
 	break;
       case Y4M_ILACE_TOP_FIRST:
-	y4m_write_fields(fdout, &sinfo, &finfo, buffers, buffers2);
+	if ((err = y4m_write_fields(fdout, &sinfo, &finfo, buffers, buffers2))
+	    != Y4M_OK)
+	  mjpeg_error_exit1("Write fields failed: %s", y4m_strerr(err));
 	break;
       case Y4M_ILACE_BOTTOM_FIRST:
-	y4m_write_fields(fdout, &sinfo, &finfo, buffers2, buffers);
+	if ((err = y4m_write_fields(fdout, &sinfo, &finfo, buffers2, buffers))
+	    != Y4M_OK)
+	  mjpeg_error_exit1("Write fields failed: %s", y4m_strerr(err));
 	break;
       default:
 	mjpeg_error_exit1("Unknown ilace type!   %d", cl.interlace);
