@@ -19,15 +19,20 @@
  *
  */
 
-
-#ifdef	HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <stdio.h>
 
 #include "macroblock.hh"
 #include "mpeg2syntaxcodes.h"
 #include "picture.hh"
 
+void MacroBlock::Encode()
+{ 
+    MotionEstimate();
+    SelectCodingModeOnVariance();
+    Predict();
+    Transform();
+}
+
 void MacroBlock::MotionEstimate()
 {
 	if (picture->pict_struct==FRAME_PICTURE)
