@@ -64,7 +64,7 @@
 
 
 
-static double scale_quantf( int q_scale_type, double quant )
+double RateCtl::ScaleQuantf( int q_scale_type, double quant )
 {
 	double quantf;
 
@@ -173,8 +173,6 @@ OnTheFlyRateCtl::OnTheFlyRateCtl(EncoderParams &encparams ) :
 	buffer_variation = 0;
 	bits_transported = 0;
 	bits_used = 0;
-	prev_bitcount = 0;
-	bitcnt_EOP = 0;
 	frame_overshoot_margin = 0;
 	sum_avg_act = 0.0;
 	sum_avg_var = 0.0;
@@ -185,12 +183,8 @@ OnTheFlyRateCtl::OnTheFlyRateCtl(EncoderParams &encparams ) :
 	   real encoding... alternative a config file should  be written!
 	*/
 
-	avg_KI = 5.0; 
-	avg_KB = 10.0*2.0;
-	avg_KP = 10.0*2.0;
 	sum_avg_quant = 0.0;
 
-
 }
 
 /*********************
@@ -198,6 +192,7 @@ OnTheFlyRateCtl::OnTheFlyRateCtl(EncoderParams &encparams ) :
  * Initialise rate control parameters
  * params:  reinit - Rate control is being re-initialised during the middle
  *                   of a run.  Don't reset adaptive parameters.
+ *
  ********************/
 
 void OnTheFlyRateCtl::InitSeq(bool reinit)
@@ -211,9 +206,7 @@ void OnTheFlyRateCtl::InitSeq(bool reinit)
 	fields_per_pict = encparams.fieldpic ? 1 : 2;
 	if( encparams.still_size > 0 )
 	{
-		avg_KI *= 1.5;
 		per_pict_bits = encparams.still_size * 8;
-		R = encparams.still_size * 8;
 	}
 	else
 	{
@@ -222,7 +215,6 @@ void OnTheFlyRateCtl::InitSeq(bool reinit)
 								 ? encparams.bit_rate / field_rate
 								 : encparams.bit_rate / encparams.decode_frame_rate
 				);
-		R = static_cast<int32_t>(encparams.bit_rate);
 	}
 
 	/* Everything else already set or adaptive */
@@ -230,6 +222,23 @@ void OnTheFlyRateCtl::InitSeq(bool reinit)
 		return;
 
 	first_gop = true;
+	K_AVG_WINDOW[I_TYPE] = 2.0;
+    switch( encparams.M )
+    {
+    case 1 : // P
+        K_AVG_WINDOW[P_TYPE] = 8.0;
+        K_AVG_WINDOW[B_TYPE] = 1.0; // dummy
+        break;
+    case 2 : // BP
+        K_AVG_WINDOW[P_TYPE] = 4.0;
+        K_AVG_WINDOW[B_TYPE] = 4.0;
+        break;
+    default: // BBP
+        K_AVG_WINDOW[P_TYPE] = 3.0;
+        K_AVG_WINDOW[B_TYPE] = 7.0;
+        break;
+    }        
+
 
 	/* Calculate reasonable margins for variation in the decoder
 	   buffer.  We assume that having less than 5 frame intervals
@@ -268,9 +277,9 @@ void OnTheFlyRateCtl::InitSeq(bool reinit)
 	  more rapidly *within* a single frame.
 	*/
 	if( encparams.still_size > 0 )
-		r = (int)floor(2.0*encparams.bit_rate/encparams.decode_frame_rate);
+		fb_gain = (int)floor(2.0*encparams.bit_rate/encparams.decode_frame_rate);
 	else
-		r = (int)floor(4.0*encparams.bit_rate/encparams.decode_frame_rate);
+		fb_gain = (int)floor(4.0*encparams.bit_rate/encparams.decode_frame_rate);
 
 
 	/* Set the virtual buffers for per-frame rate control feedback to
@@ -279,22 +288,14 @@ void OnTheFlyRateCtl::InitSeq(bool reinit)
 	*/
 
 	init_quant = (encparams.quant_floor > 0.0 ? encparams.quant_floor : 6.0);
-	d0p = d0b = d0pb = d0i = static_cast<int>(init_quant * r / 62.0);
+    int i;
+    for( i = FIRST_PICT_TYPE; i <= LAST_PICT_TYPE; ++i )
+        ratectl_vbuf[i] = static_cast<int>(init_quant * fb_gain / 62.0);
 
 	next_ip_delay = 0.0;
 	decoding_time = 0.0;
 
 
-#ifdef OUTPUTr_STAT
-	fprintf(statfile,"\nrate control: sequence initialization\n");
-	fprintf(statfile,
-			" initial global complexity measures (I,P,B): Xi=%.0f, Xp=%.0f, Xb=%.0f\n",
-			Xi, Xp, Xb);
-	fprintf(statfile," reaction parameter: r=%d\n", r);
-	fprintf(statfile,
-			" initial virtual buffer fullness (I,P,B): d0i=%d, d0pb=%d",
-			d0i, d0pb);
-#endif
 
 
 }
@@ -302,10 +303,10 @@ void OnTheFlyRateCtl::InitSeq(bool reinit)
 
 void OnTheFlyRateCtl::InitGOP( int np, int nb)
 {
-	Np = encparams.fieldpic ? 2*np+1 : 2*np;
-	Nb = encparams.fieldpic ? 2*nb : 2*nb;
-	Ni = encparams.fieldpic ? 1 : 2;
-	fields_in_gop = Ni + Nb + Np;
+	N[P_TYPE] = encparams.fieldpic ? 2*np+1 : 2*np;
+	N[B_TYPE] = encparams.fieldpic ? 2*nb : 2*nb;
+	N[I_TYPE] = encparams.fieldpic ? 1 : 2;
+	fields_in_gop = N[I_TYPE] + N[P_TYPE] + N[B_TYPE];
 
 	/*
 	  At the start of a GOP before any frames have gone the
@@ -320,52 +321,48 @@ void OnTheFlyRateCtl::InitGOP( int np, int nb)
 	   for each one.  They're all I-frames so each stills is a GOP too.
 	*/
 
+    int i;
 	if( first_gop || encparams.still_size > 0)
 	{
 		mjpeg_debug( "FIRST GOP INIT");
 		fast_tune = true;
-		first_I = first_B = first_P = true;
 		first_gop = false;
-		I_pict_base_bits = per_pict_bits;
-		B_pict_base_bits = per_pict_bits;
-		P_pict_base_bits = per_pict_bits;
+        for( i = FIRST_PICT_TYPE; i <= LAST_PICT_TYPE; ++i )
+        {
+            first_encountered[i] = true;
+            pict_base_bits[i] = per_pict_bits;
+        }
 	}
 	else
 	{
+		mjpeg_debug( "REST GOP INIT" );
 		double recovery_fraction = field_rate/(overshoot_gain * fields_in_gop);
 		double recovery_gain = 
 			recovery_fraction > 1.0 ? 1.0 : overshoot_gain * recovery_fraction;
 		int available_bits = 
 			static_cast<int>( (encparams.bit_rate+buffer_variation*recovery_gain)
 							  * fields_in_gop/field_rate);
-		double Xsum = Ni*Xi+Np*Xp+Nb*Xb;
-		mjpeg_debug( "REST GOP INIT" );
-		I_pict_base_bits = (int32_t)(fields_per_pict*available_bits*Xi/Xsum);
-		P_pict_base_bits = (int32_t)(fields_per_pict*available_bits*Xp/Xsum);
-		B_pict_base_bits = (int32_t)(fields_per_pict*available_bits*Xb/Xsum);
-		fast_tune = 0;
+        double Xsum = 0.0;
+        for( i = FIRST_PICT_TYPE; i <= LAST_PICT_TYPE; ++i )
+            Xsum += N[i]*Xhi[i];
+        for( i = FIRST_PICT_TYPE; i <= LAST_PICT_TYPE; ++i )
+            pict_base_bits[i] =  
+                static_cast<int32_t>(fields_per_pict*available_bits*Xhi[i]/Xsum);
+		fast_tune = false;
 
 	}
 
-#ifdef OUTPUT_STAT
-	fprintf(statfile,"\nrate control: new group of pictures (GOP)\n");
-	fprintf(statfile," target number of bits for GOP: R=%.0f\n",R);
-	fprintf(statfile," number of P pictures in GOP: Np=%d\n",Np);
-	fprintf(statfile," number of B pictures in GOP: Nb=%d\n",Nb);
-#endif
 }
 
 
-/* Step 1: compute target bits for current picture being coded */
-void OnTheFlyRateCtl::InitPict(Picture &picture, int64_t bitcount_SOP)
+/* Step 1a: compute target bits for current picture being coded, based
+ * on predicting from past frames */
+
+void OnTheFlyRateCtl::InitNewPict(Picture &picture)
 {
-	double avg_K = 0.0;
 	double target_Q;
-	double current_Q;
-	double Si, Sp, Sb;
 	int available_bits;
 	double Xsum,varsum;
-	bool no_avg_K = false;
 
 	/* TODO: A.Stevens  Nov 2000 - This modification needs testing visually.
 
@@ -380,7 +377,7 @@ void OnTheFlyRateCtl::InitPict(Picture &picture, int64_t bitcount_SOP)
 	   of DCT coefficients) and actual quantisation weighted activty.
 	   We use this to try to predict the activity of each frame.
 	*/
-	
+
 	picture.ActivityMeasures( actsum, varsum );
 	avg_act = actsum/(double)(encparams.mb_per_pict);
 	avg_var = varsum/(double)(encparams.mb_per_pict);
@@ -394,9 +391,8 @@ void OnTheFlyRateCtl::InitPict(Picture &picture, int64_t bitcount_SOP)
 	   weighted by:
 	   - global complexity averages
 	   - predicted activity measures
-	   - fixed type based weightings
 	   
-	   T = (Nx * Xx/Kx) / Sigma_j (Nj * Xj / Kj)
+	   T = available_bits * (Nx * Xx) / Sigma_j (Nj * Xj)
 
 	   N.b. B frames are an exception as there is *no* predictive
 	   element in their bit-allocations.  The reason this is done is
@@ -408,8 +404,6 @@ void OnTheFlyRateCtl::InitPict(Picture &picture, int64_t bitcount_SOP)
 	   In the case of scene changes similar considerations apply.  In
 	   this case also we want to save bits for the next I or P frame
 	   where they will help improve other frames too.  
-	   TODO: Experiment with *inverse* predictive correction for B-frames
-	   and turning off active-block boosting for B-frames.
 
 	   Note that we have to calulate per-frame bits by scaling the one-second
 	   bit-pool a one-GOP bit-pool.
@@ -432,67 +426,28 @@ void OnTheFlyRateCtl::InitPict(Picture &picture, int64_t bitcount_SOP)
 							  );
 	}
 
-	min_q = min_d = INT_MAX;
-	max_q = max_d = INT_MIN;
-    Xsum = Ni*Xi+Np*Xp+Nb*Xb;
-	switch (picture.pict_type)
-	{
-	case I_TYPE:
-		
-		/* There is little reason to rely on the *last* I-frame as
-		   they're not closely related.  The slow correction of K
-		   should be enough to fine-tune.  
-		*/
+    Xsum = 0.0;
+    int i;
+    for( i = FIRST_PICT_TYPE; i <= LAST_PICT_TYPE; ++i )
+        Xsum += N[i]*Xhi[i];
+    vbuf_fullness = ratectl_vbuf[picture.pict_type];
+    double first_weight[NUM_PICT_TYPES];
+    first_weight[I_TYPE] = 1.0;
+    first_weight[P_TYPE] = 1.7;
+    first_weight[B_TYPE] = 1.7*2.0;
 
-		d = d0i;
-		avg_K = avg_KI;
-		Si = avg_K*actsum;
-		no_avg_K = first_I;
-		if( first_I )
-		{
-			T = (int32_t)(fields_per_pict*available_bits/(Ni+(Np/1.7)+Nb/(2.0*1.7)));
-		}
-		else
-		{
-			T = (int32_t)(fields_per_pict*available_bits*Si/Xsum);
-		}
-		pict_base_bits = I_pict_base_bits;
-		break;
-	case P_TYPE:
-		d = d0p;
-		avg_K = avg_KP;
-		Sp = (2*Xp + avg_K * actsum)/3.0;	 /* Damp P-frame response a little
-											as really big jumps will probably
-											make avg_K inaccurate */
-		no_avg_K = first_P;
-		if( first_P )
-		{
-			T = (int32_t)(fields_per_pict*available_bits/(Np+Nb/2.0));
-		}
-		else
-		{
-			T = (int32_t)(fields_per_pict*available_bits*Sp/Xsum);
-		}
-		pict_base_bits = P_pict_base_bits;
-		break;
-	case B_TYPE:
-		d = d0b;
-		avg_K = avg_KB;
-		Sb = Xb;				/* We don't want B-frames to be too
-								   responsive...*/
-		no_avg_K = first_B;
-		if( first_B )
-		{
-			T =  (int32_t)(fields_per_pict*available_bits/(Nb+Np*2.0));
-		}
-		else
-		{
-			T =  (int32_t)(fields_per_pict*available_bits*Sb/Xsum);
-		}
-		pict_base_bits = B_pict_base_bits;
-
-		break;
-	}
+    double gop_frac = 0.0;
+    if( first_encountered[picture.pict_type] )
+    {
+        for( i = FIRST_PICT_TYPE; i <= LAST_PICT_TYPE; ++i )
+            gop_frac += N[i]/first_weight[i];
+        target_bits = 
+            static_cast<int32_t>(fields_per_pict*available_bits /
+                                 (gop_frac * first_weight[picture.pict_type]));
+    }
+    else
+        target_bits =
+            static_cast<int32_t>(fields_per_pict*available_bits*Xhi[picture.pict_type]/Xsum);
 
 	/* 
 	   If we're fed a sequences of identical or near-identical images
@@ -501,17 +456,19 @@ void OnTheFlyRateCtl::InitPict(Picture &picture, int64_t bitcount_SOP)
 	   limit any individual frame to 3/4's of the buffer.
 	*/
 
-	T = intmin( T, encparams.video_buffer_size*3/4 );
+	target_bits = min( target_bits, encparams.video_buffer_size*3/4 );
 
-	mjpeg_debug( "I=%d P=%d B=%d", I_pict_base_bits, P_pict_base_bits, B_pict_base_bits );
-	mjpeg_debug( "T=%05d A=%06d D=%06d (%06d) ", (int)T/8, (int)available_bits/8, (int)buffer_variation/8, (int)(buffer_variation + gop_buffer_correction)/8 );
+ 	mjpeg_debug( "Frame %c T=%05d A=%06d  Xi=%.2f Xp=%.2f Xb=%.2f", 
+                 pict_type_char[picture.pict_type],
+                 (int)target_bits/8, (int)available_bits/8, 
+                 Xhi[I_TYPE], Xhi[P_TYPE],Xhi[B_TYPE] );
 
 
 	/* 
 	   To account for the wildly different sizes of frames
 	   we compute a correction to the current instantaneous
 	   buffer state that accounts for the fact that all other
-	   thing being equal buffer to go down a lot after the I-frame
+	   thing being equal buffer will go down a lot after the I-frame
 	   decode but fill up again through the B and P frames.
 
 	   For this we use the base bit allocations of the picture's
@@ -520,66 +477,136 @@ void OnTheFlyRateCtl::InitPict(Picture &picture, int64_t bitcount_SOP)
 	   bit-allocation (which *won't* add up very well).
 	*/
 
-	mjpeg_debug( "PBB=%d PPB=%d", pict_base_bits, per_pict_bits );
-	gop_buffer_correction += (pict_base_bits-per_pict_bits);
+	gop_buffer_correction += (pict_base_bits[picture.pict_type]-per_pict_bits);
 
 
 	/* Undershot bits have been "returned" via R */
-	if( d < 0 )
-		d = 0;
+    vbuf_fullness = max( vbuf_fullness, 0 );
 
 	/* We don't let the target volume get absurdly low as it makes some
 	   of the prediction maths ill-condtioned.  At these levels quantisation
 	   is always minimum anyway
 	*/
-	T = intmax( T, 4000 );
+	target_bits = max( target_bits, 4000 );
 
 	if( encparams.still_size > 0 && encparams.vbv_buffer_still_size )
 	{
 		/* If stills size must match then target low to ensure no
 		   overshoot.
 		*/
-		mjpeg_info( "Setting VCD HR still overshoot margin to %d bytes", T/(16*8) );
-		frame_overshoot_margin = T/16;
-		T -= frame_overshoot_margin;
+		mjpeg_info( "Setting VCD HR still overshoot margin to %d bytes", target_bits/(16*8) );
+		frame_overshoot_margin = target_bits/16;
+		target_bits -= frame_overshoot_margin;
 	}
 
 
-	current_Q = ScaleQuant(picture.q_scale_type,62.0*d / r);
-	if( no_avg_K )
-		target_Q = current_Q;
-	else
-		target_Q = scale_quantf(picture.q_scale_type, 
-								avg_K * avg_act *(encparams.mb_per_pict) / T);
-
 	picture.avg_act = avg_act;
 	picture.sum_avg_act = sum_avg_act;
+    mquant_change_ctr = encparams.mb_width;
+	cur_mquant = ScaleQuant( picture.q_scale_type, 
+                             fmax( vbuf_fullness*62.0/fb_gain, encparams.quant_floor) );
+    mquant_change_ctr = encparams.mb_width;
+}
 
-	/*  If guesstimated target quantisation requirement is much different
-		than that suggested by the virtual buffer we reset the virtual
-		buffer to set the guesstimate as initial quantisation.
+/* Step 1b: compute target bits for current picture being coded, based
+ * on an actual encoding */
 
-		This effectively speeds up rate control response when sudden
-		changes in content occur
+void OnTheFlyRateCtl::InitKnownPict( Picture &picture )
+{
+	double target_Q;
+	int available_bits;
+	double Xsum,varsum;
+
+	actcovered = 0.0;
+	sum_vbuf_Q = 0.0;
+
+	/* Allocate target bits for frame based on frames numbers in GOP
+	   weighted by:
+	   - global complexity averages
+	   - predicted activity measures
+	   
+	   T = (Nx * Xx) / Sigma_j (Nj * Xj)
 	*/
 
-
-	if ( 62.0*d / r  < target_Q / 2.0  )
+	
+	if( encparams.still_size > 0 )
+		available_bits = per_pict_bits;
+	else
 	{
-		d = (int) (target_Q  * r / (62.0));
+		int feedback_correction =
+			static_cast<int>( fast_tune 
+							  ?	buffer_variation * overshoot_gain
+							  : (buffer_variation+gop_buffer_correction) 
+							    * overshoot_gain
+				);
+		available_bits = 
+			static_cast<int>( (encparams.bit_rate+feedback_correction)
+							  * fields_in_gop/field_rate
+							  );
 	}
 
-	if ( 62.0*d / r  > target_Q * 2.0  )
+    Xsum = 0.0;
+    int i;
+    double rawquant = InvScaleQuant( picture.q_scale_type, 
+                                     static_cast<int>(actual_avg_Q) );
+    vbuf_fullness = static_cast<int>(rawquant*fb_gain/62.0);
+    for( i = FIRST_PICT_TYPE; i <= LAST_PICT_TYPE; ++i )
+        Xsum += N[i]*Xhi[i];
+    target_bits =
+        static_cast<int32_t>(fields_per_pict*available_bits*actual_Xhi/Xsum);
+
+	/* 
+	   If we're fed a sequences of identical or near-identical images
+	   we can get actually get allocations for frames that exceed
+	   the video buffer size!  This of course won't work so we arbitrarily
+	   limit any individual frame to 3/4's of the buffer.
+	*/
+
+	target_bits = min( target_bits, encparams.video_buffer_size*3/4 );
+
+	mjpeg_debug( "Frame %c T=%05d A=%06d  Xi=%.2f Xp=%.2f Xb=%.2f", 
+                 pict_type_char[picture.pict_type],
+                 (int)target_bits/8, (int)available_bits/8, 
+                 Xhi[I_TYPE], Xhi[P_TYPE],Xhi[B_TYPE] );
+
+
+	/* 
+	   To account for the wildly different sizes of frames
+	   we compute a correction to the current instantaneous
+	   buffer state that accounts for the fact that all other
+	   thing being equal buffer will go down a lot after the I-frame
+	   decode but fill up again through the B and P frames.
+
+	   For this we use the base bit allocations of the picture's
+	   "pict_base_bits" which will pretty accurately add up to a
+	   GOP-length's of bits not the more dynamic predictive T target
+	   bit-allocation (which *won't* add up very well).
+	*/
+
+	gop_buffer_correction += (pict_base_bits[picture.pict_type]-per_pict_bits);
+
+
+	/* We don't let the target volume get absurdly low as it makes some
+	   of the prediction maths ill-condtioned.  At these levels quantisation
+	   is always minimum anyway
+	*/
+	target_bits = max( target_bits, 4000 );
+
+	if( encparams.still_size > 0 && encparams.vbv_buffer_still_size )
 	{
-		d = (int) (d+(target_Q  * r / 62.0))/2;
+		/* If stills size must match then target low to ensure no
+		   overshoot.
+		*/
+		mjpeg_info( "Setting VCD HR still overshoot margin to %d bytes", target_bits/(16*8) );
+		frame_overshoot_margin = target_bits/16;
+		target_bits -= frame_overshoot_margin;
 	}
 
-	S = bitcount_SOP;
-
-#ifdef OUTPUT_STAT
-	fprintf(statfile,"\nrate control: start of picture\n");
-	fprintf(statfile," target number of bits: T=%.0f\n",T/8);
-#endif
+    printf( "vbuf = %d\n", vbuf_fullness );
+	cur_mquant = ScaleQuant( picture.q_scale_type, 
+                             fmax( vbuf_fullness*62.0/fb_gain, encparams.quant_floor) );
+    printf( "MQ = %d\n", cur_mquant );
+    mquant_change_ctr = encparams.mb_width;
 }
 
 
@@ -591,25 +618,21 @@ void OnTheFlyRateCtl::InitPict(Picture &picture, int64_t bitcount_SOP)
  * rate constraints...
  */
 
-int OnTheFlyRateCtl::UpdatePict(Picture &picture, int64_t _bitcount_EOP)
+void OnTheFlyRateCtl::UpdatePict( Picture &picture, int &padding_needed)
 {
-	double X;
 	double K;
-	double AQ;
-	int32_t AP;		/* Actual (including padding) picture bit counts */
+	int32_t actual_bits;		/* Actual (inc. padding) picture bit counts */
 	int    i;
 	int    Qsum;
 	int frame_overshoot;
-    int64_t bitcount_EOP = _bitcount_EOP; 
-	AP = bitcount_EOP - S;
-	frame_overshoot = (int)AP-(int)T;
-
+	actual_bits = picture.SizeCodedMacroBlocks();
+	frame_overshoot = (int)actual_bits-(int)target_bits;
 	/* For the virtual buffers for quantisation feedback it is the
 	   actual under/overshoot *including* padding.  Otherwise the
 	   buffers go zero.
        BUGBUGBUG  should'nt this go after the padding calculation?
 	*/
-	d += frame_overshoot;
+	vbuf_fullness += frame_overshoot;
 
 	/* Warn if it looks like we've busted the safety margins in stills
 	   size specification.  Adjust padding to account for safety
@@ -623,7 +646,7 @@ int OnTheFlyRateCtl::UpdatePict(Picture &picture, int64_t _bitcount_EOP)
 		if( frame_overshoot > frame_overshoot_margin )
 		{
 			mjpeg_warn( "Rate overshoot: VCD hi-res still %d bytes too large! ", 
-						((int)AP)/8-encparams.still_size);
+						((int)actual_bits)/8-encparams.still_size);
 		}
 		
 		//
@@ -638,27 +661,15 @@ int OnTheFlyRateCtl::UpdatePict(Picture &picture, int64_t _bitcount_EOP)
         // Make sure we pad nicely to byte alignment
         if( frame_overshoot < 0 )
         {
-            padding_bits = (((bitcount_EOP-frame_overshoot)>>3)<<3)-bitcount_EOP;
+            padding_bits = (((actual_bits-frame_overshoot)>>3)<<3)-actual_bits;
             picture.pad = 1;
         }
-#ifdef JUNK_DONE_ELSEWHERE_NOW
-		if( padding_bytes > 0 )
-		{
-			mjpeg_debug( "Padding still to size: %d extra bytes", padding_bytes );
-			picture.pad = 1;
-			for( i = 0; i < padding_bytes/2; ++i )
-			{
-				writer.PutBits(0, 16);
-			}
-		}
-#endif
 	}
 
     /* Adjust the various bit counting  parameters for the padding bytes that
      * will be added */
-    AP += padding_bits ;
+    actual_bits += padding_bits ;
     frame_overshoot += padding_bits;
-    bitcount_EOP += padding_bits;
 
 	/*
 	  Compute the estimate of the current decoder buffer state.  We
@@ -674,12 +685,11 @@ int OnTheFlyRateCtl::UpdatePict(Picture &picture, int64_t _bitcount_EOP)
 	*/
 
 	
-	bits_used += (bitcount_EOP-prev_bitcount);
-	prev_bitcount = bitcount_EOP;
-    
+	bits_used += actual_bits;
+ 
 	bits_transported += per_pict_bits;
-	mjpeg_debug( "TR=%" PRId64 " USD=%" PRId64 "", bits_transported/8, bits_used/8);
-	buffer_variation  = (int32_t)(bits_transported - bits_used);
+	//mjpeg_debug( "TR=%" PRId64 " USD=%" PRId64 "", bits_transported/8, bits_used/8);
+	buffer_variation  = static_cast<int32_t>(bits_transported - bits_used);
 
 	if( buffer_variation > 0 )
 	{
@@ -695,7 +705,6 @@ int OnTheFlyRateCtl::UpdatePict(Picture &picture, int64_t _bitcount_EOP)
 		}
 	}
 
-
 	Qsum = 0;
 	for( i = 0; i < encparams.mb_per_pict; ++i )
 	{
@@ -703,12 +712,12 @@ int OnTheFlyRateCtl::UpdatePict(Picture &picture, int64_t _bitcount_EOP)
 	}
 
 	
-    /* AQ is the average Quantisation of the block.
+    /* actual_Q is the average Quantisation of the block.
 	   Its only used for stats display as the integerisation
 	   of the quantisation value makes it rather coarse for use in
 	   estimating bit-demand */
-	AQ = (double)Qsum/(double)encparams.mb_per_pict;
-	sum_avg_quant += AQ;
+	actual_avg_Q  = static_cast<double>(Qsum)/encparams.mb_per_pict;
+	sum_avg_quant += actual_avg_Q;
 	
 	/* X (Chi - Complexity!) is an estimate of "bit-demand" for the
 	frame.  I.e. how many bits it would need to be encoded without
@@ -721,15 +730,21 @@ int OnTheFlyRateCtl::UpdatePict(Picture &picture, int64_t _bitcount_EOP)
 	prediction of quantisation needed to hit a bit-allocation.
 	*/
 
-	X = AP  * AQ; //sum_vbuf_Q /  mb_per_pict;
-	K = X / actsum;
-	picture.AQ = AQ;
+	actual_Xhi = actual_bits * actual_avg_Q;
+
+    /* To handle longer sequences with little picture content
+       where I, B and P frames are of unusually similar size we
+       insist I frames assumed to be at least one and a half times
+       as complex as typical P frames
+    */
+    if( picture.pict_type == I_TYPE )
+        actual_Xhi = fmax(actual_Xhi, 1.5*Xhi[P_TYPE]);
+
+	picture.AQ = actual_avg_Q;
 	picture.SQ = sum_avg_quant;
 
-	mjpeg_debug( "D=%d R=%d GC=%d", 
-				 buffer_variation/8,
-				 (int)R/8,
-				 gop_buffer_correction/8  );
+	//mjpeg_debug( "D=%d R=%d GC=%d", buffer_variation/8, (int)R/8, 
+    //gop_buffer_correction/8  );
 
 	/* Xi are used as a guesstimate of *typical* frame activities
 	   based on the past.  Thus we don't want anomalous outliers due
@@ -739,90 +754,37 @@ int OnTheFlyRateCtl::UpdatePict(Picture &picture, int64_t _bitcount_EOP)
 	   similar real-time decay periods based on an assumption of
 	   20-30Hz frame rates.
 	*/
+    
+    ratectl_vbuf[picture.pict_type] = vbuf_fullness;
+    sum_size[picture.pict_type] += actual_bits/8.0;
+    pict_count[picture.pict_type] += 1;
 
-	switch (picture.pict_type)
-	{
-	case I_TYPE:
-		d0i = d;
-		if( first_I )
-		{
-			Xi = X;
-			avg_KI = K;
-			first_I = 0;
-		}
-		else
-		{
-			avg_KI = (K + avg_KI * K_AVG_WINDOW_I) / (K_AVG_WINDOW_I+1.0) ;
-			Xi = (X + K_AVG_WINDOW_I*Xi)/(K_AVG_WINDOW_I+1.0);
+    if( first_encountered[picture.pict_type] )
+    {
+        Xhi[picture.pict_type] = actual_Xhi;
+        first_encountered[picture.pict_type] = false;
+    }
+    else
+    {
+        double win = fast_tune 
+            ? K_AVG_WINDOW[picture.pict_type] / 1.7
+            : K_AVG_WINDOW[picture.pict_type];
+        Xhi[picture.pict_type] = 
+            (actual_Xhi + win*Xhi[picture.pict_type])/(win+1.0);
+    }
 
-			/* To handle longer sequences with little picture content
-		   where I, B and P frames are of unusually similar size we
-		   insist I frames assumed to be at least of the same
-		   complexity as two B and a P frame.  This ensures that
-		   *after* a low picture content sequence there won't be a
-		   grossly under-estimated I-frame size that hasn't been
-		   allowed for in the buffer management.
-			*/
-			Xi = Xi < Xp+2.0*Xb ? Xp+2.0*Xb  : Xi;
-		}
-		break;
-	case P_TYPE:
-		d0p = d;
-		if( first_P )
-		{
-			Xp = X;
-			avg_KP = K;
-			first_P = 0;
-		}
-		else
-		{
-			avg_KP = (K + avg_KP * K_AVG_WINDOW_P) / (K_AVG_WINDOW_P+1.0) ;
-			if( fast_tune )
-				Xp = (X+Xp*2.0)/3.0;
-			else
-				Xp = (X + Xp*K_AVG_WINDOW_P)/(K_AVG_WINDOW_P+1.0);
-		}
-		break;
-	case B_TYPE:
-		d0b = d;
-		if( first_B )
-		{
-			Xb = X;
-			avg_KB = K;
-			first_B = 0;
-		}
-		else 
-		{
-			avg_KB = (K + avg_KB * K_AVG_WINDOW_B) / (K_AVG_WINDOW_B+1.0) ;
-			if( fast_tune )
-			{
-				Xb = (X + Xb * 3.0) / 4.0;
-			}
-			else
-				Xb = (X + Xb*K_AVG_WINDOW_B)/(K_AVG_WINDOW_B+1.0);
-		}
-		break;
-	}
-
-	VbvEndOfPict(picture, bitcount_EOP);
-
-#ifdef OUTPUT_STAT
-	fprintf(statfile,"\nrate control: end of picture\n");
-	fprintf(statfile," actual number of bits: S=%lld\n",S);
-	fprintf(statfile," average quantization parameter AQ=%.1f\n",
-			(double)AQ);
-	fprintf(statfile," remaining number of bits in GOP: R=%.0f\n",R);
-	fprintf(statfile,
-			" global complexity measures (I,P,B): Xi=%.0f, Xp=%.0f, Xb=%.0f\n",
-			Xi, Xp, Xb);
-	fprintf(statfile,
-			" virtual buffer fullness (I,PB): d0i=%d, d0b=%d\n",
-			d0i, d0b);
-	fprintf(statfile," remaining number of P pictures in GOP: Np=%d\n",Np);
-	fprintf(statfile," remaining number of B pictures in GOP: Nb=%d\n",Nb);
-	fprintf(statfile," average activity: avg_act=%.1f \n", avg_act );
-#endif
-    return padding_bits/8;
+    mjpeg_debug( "Frame %c A=%6.0f %.2f: I = %6.0f P = %5.0f B = %5.0f",
+                 pict_type_char[picture.pict_type],
+                 actual_bits/8.0,
+                 actual_Xhi,
+                 sum_size[I_TYPE]/pict_count[I_TYPE],
+                 sum_size[P_TYPE]/pict_count[P_TYPE],
+                 sum_size[B_TYPE]/pict_count[B_TYPE]
+        );
+    
+                
+	VbvEndOfPict(picture);
+    padding_needed = padding_bits/8;
 }
 
 /* compute initial quantization stepsize (at the beginning of picture) 
@@ -833,14 +795,7 @@ int OnTheFlyRateCtl::UpdatePict(Picture &picture, int64_t _bitcount_EOP)
 
 int OnTheFlyRateCtl::InitialMacroBlockQuant(Picture &picture)
 {
-	
-	int mquant = ScaleQuant( picture.q_scale_type, d*62.0/r );
-	
-/*
-  fprintf(statfile,"rc_start_mb:\n");
-  fprintf(statfile,"mquant=%d\n",mquant);
-*/
-	return intmax(mquant, static_cast<int>(encparams.quant_floor));
+    return cur_mquant;
 }
 
 
@@ -849,89 +804,81 @@ int OnTheFlyRateCtl::InitialMacroBlockQuant(Picture &picture)
  * SelectQuantization - select a quantisation for the current
  * macroblock based on the fullness of the virtual decoder buffer.
  *
+ * NOTE: *Must* be called for all Macroblocks as content-based quantisation tuning is
+ * supported.
  ************/
 
-int OnTheFlyRateCtl::MacroBlockQuant( const MacroBlock &mb, int64_t bitcount )
+int OnTheFlyRateCtl::MacroBlockQuant( const MacroBlock &mb )
 {
-	int mquant;
-	int lum_variance = mb.BaseLumVariance();
-	double act  = mb.Activity();
-	const Picture &picture = mb.ParentPicture();
-	/* A.Stevens 2000 : we measure how much *information* (total activity)
-	   has been covered and aim to release bits in proportion.
+    --mquant_change_ctr;
+    if( mquant_change_ctr < 0)
+        mquant_change_ctr =  encparams.mb_width/2;
+    int lum_variance = mb.BaseLumVariance();
+    int mquant;
+    if( mquant_change_ctr == 0 || lum_variance < encparams.boost_var_ceil )
+    {
+        const Picture &picture = mb.ParentPicture();
+        /* A.Stevens 2000 : we measure how much *information* (total activity)
+           has been covered and aim to release bits in proportion.
 
-	   We keep track of a virtual buffer that catches the difference
-	   between the bits allocated and the bits we actually used.  The
-	   fullness of this buffer controls quantisation.
+           We keep track of a virtual buffer that catches the difference
+           between the bits allocated and the bits we actually used.  The
+           fullness of this buffer controls quantisation.
 
-	*/
+        */
 
-	/* Guesstimate a virtual buffer fullness based on
-	   bits used vs. bits in proportion to activity encoded
-	*/
+        /* Guesstimate a virtual buffer fullness based on
+           bits used vs. bits in proportion to activity encoded
+        */
 
-	double dj = ((double)d) + 
-		((double)(bitcount-S) - actcovered * ((double)T) / actsum);
+        double dj = static_cast<double>(vbuf_fullness) 
+            + static_cast<double>(picture.SizeCodedMacroBlocks()) 
+            - actcovered * target_bits / actsum;
 
 
-	/* scale against dynamic range of mquant and the bits/picture
-	   count.  encparams.quant_floor != 0.0 is the VBR case where we set a
-	   bitrate as a (high) maximum and then put a floor on
-	   quantisation to achieve a reasonable overall size.  Not that
-	   this *is* baseline quantisation.  Not adjust for local
-	   activity.  Otherwise we end up blurring active
-	   macroblocks. Silly in a VBR context.
-	*/
+        /* scale against dynamic range of mquant and the bits/picture
+           count.  encparams.quant_floor != 0.0 is the VBR case where we set a
+           bitrate as a (high) maximum and then put a floor on
+           quantisation to achieve a reasonable overall size.  Not that
+           this *is* baseline quantisation.  Not adjust for local
+           activity.  Otherwise we end up blurring active
+           macroblocks. Silly in a VBR context.
+        */
 
-	double Qj = dj*62.0/r;
-	Qj = (Qj > encparams.quant_floor) ? Qj : encparams.quant_floor;
+        double Qj = dj*62.0/fb_gain;
+        Qj =  fmax(Qj,encparams.quant_floor);
 
-	/*  Heuristic: We decrease quantisation for macroblocks
-		with markedly low luminace variance.  This helps make
-		gentle gradients (e.g. smooth backgrounds) look better at
-		(hopefully) small additonal cost  in coding bits
-	*/
+        /*  Heuristic: We decrease quantisation for macroblocks
+            with markedly low luminace variance.  This helps make
+            gentle gradients (e.g. smooth backgrounds) look better at
+            (hopefully) small additonal cost  in coding bits
+        */
 
-	double act_boost;
-#ifdef OLD_QUANTISATION_STEARING
-	double N_act =  ( act < avg_act || picture.pict_type == B_TYPE ) ? 
-		1.0 : 
-		(encparams.act_boost*act +  avg_act)/(act + encparams.act_boost*avg_act);
-	act_boost = 1.0/N_act;
-#else
-	if( lum_variance < encparams.boost_var_ceil )
-	{
-		if( lum_variance < encparams.boost_var_ceil/2)
-			act_boost = encparams.act_boost;
-		else
-		{
-			double max_boost_var = encparams.boost_var_ceil/2;
-			double above_max_boost = 
-				(static_cast<double>(lum_variance)-max_boost_var)
-				/ max_boost_var;
-			act_boost = 1.0 + (encparams.act_boost-1.0) * (1.0-above_max_boost);
-		}
-	}
-	else
-		act_boost = 1.0;
+        double act_boost;
+        if( lum_variance < encparams.boost_var_ceil )
+        {
+            if( lum_variance < encparams.boost_var_ceil/2)
+                act_boost = encparams.act_boost;
+            else
+            {
+                double max_boost_var = encparams.boost_var_ceil/2;
+                double above_max_boost = 
+                    (static_cast<double>(lum_variance)-max_boost_var)
+                    / max_boost_var;
+                act_boost = 1.0 + (encparams.act_boost-1.0) * (1.0-above_max_boost);
+            }
+        }
+        else
+            act_boost = 1.0;
+        sum_vbuf_Q += ScaleQuantf(picture.q_scale_type,Qj/act_boost);
+        cur_mquant = ScaleQuant(picture.q_scale_type,Qj/act_boost) ;
+    }
 
-#endif	
-	sum_vbuf_Q += scale_quantf(picture.q_scale_type,Qj/act_boost);
-	mquant = ScaleQuant(picture.q_scale_type,Qj/act_boost) ;
-	
 	/* Update activity covered */
-
+	double act  = mb.Activity();
 	actcovered += act;
 	 
-	return mquant;
-#ifdef OUTPUT_STAT
-/*
-  fprintf(statfile,"MQ(%d): ",j);
-  fprintf(statfile,"dj=%.0f, Qj=%1.1f, actj=3.1%f, N_actj=1.1%f, mquant=%03d\n",
-  dj,Qj,actj,N_actj,mquant);
-*/
-	//picture.mbinfo[j].N_act = N_actj;
-#endif
+	return cur_mquant;
 }
 
 /* VBV calculations
@@ -949,10 +896,8 @@ int OnTheFlyRateCtl::MacroBlockQuant( const MacroBlock &mb, int64_t bitcount )
  * bit-stream.
  */
 
-void OnTheFlyRateCtl::VbvEndOfPict(Picture &picture, int64_t bitcount)
+void OnTheFlyRateCtl::VbvEndOfPict(Picture &picture)
 {
-	bitcnt_EOP = bitcount - BITCOUNT_OFFSET;
-
 }
 
 /* calc_vbv_delay
@@ -1108,11 +1053,6 @@ void OnTheFlyRateCtl::CalcVbvDelay(Picture &picture)
 	}
 
 
-#ifdef OUTPUT_STAT
-	fprintf(statfile,
-			"\nvbv_delay=%d (coder.BitCount=%lld, decoding_time=%.2f, bitcnt_EOP=%lld)\n",
-			picture.vbv_delay,coder.BitCount(),decoding_time,bitcnt_EOP);
-#endif
 
 	if (picture.vbv_delay<0)
 	{
