@@ -23,11 +23,7 @@
 #include <config.h>
 #include <stdlib.h>
 #include <ctype.h>
-#ifdef _WIN32
-#include <win32defs.h>
-#else
 #include <unistd.h>
-#endif
 #include <sys/stat.h>
 #include <mjpeg_logging.h>
 #include <format_codes.h>
@@ -131,6 +127,18 @@ void MultiplexJob::SetupInputStreams( std::vector< IBitStream *> &inputs )
         bs = inputs[i];
         // Remember the streams initial state...
         bs->PrepareUndo( undo );
+        if( LPCMStream::Probe( *bs ) )
+        {
+            mjpeg_info ("File %s looks like an LPCM Audio stream.",
+                        bs->StreamName());
+            bs->UndoChanges( undo );
+            streams.push_back( new JobStream( bs,  LPCM_AUDIO) );
+            ++audio_tracks;
+            ++lpcm_tracks;
+            continue;
+        }
+
+        bs->UndoChanges( undo );
         if( MPAStream::Probe( *bs ) )
         {
             mjpeg_info ("File %s looks like an MPEG Audio stream.", 
@@ -163,19 +171,7 @@ void MultiplexJob::SetupInputStreams( std::vector< IBitStream *> &inputs )
             continue;
         }
 
-        bs->UndoChanges( undo);
-        if( LPCMStream::Probe( *bs ) )
-        {
-            mjpeg_info ("File %s looks like an LPCM Audio stream.",
-                        bs->StreamName());
-            bs->UndoChanges( undo );
-            streams.push_back( new JobStream( bs,  LPCM_AUDIO) );
-            ++audio_tracks;
-            ++lpcm_tracks;
-            continue;
-        }
         bs->UndoChanges( undo );
-
         if( VideoStream::Probe( *bs ) )
         {
             mjpeg_info ("File %s looks like an MPEG Video stream.",
@@ -200,8 +196,8 @@ void MultiplexJob::SetupInputStreams( std::vector< IBitStream *> &inputs )
         }
 #endif
         bad_file = true;
-        delete bs;
         mjpeg_error ("File %s unrecogniseable!", bs->StreamName());
+        delete bs;
     }
     
     if( bad_file )
