@@ -99,7 +99,7 @@ void ZAlphaStream::ScanFirstSeqHeader()
 		exit (1);
     }
 
-	if (picture_rate >0 && picture_rate <= mpeg_num_framerates)
+	if (mpeg_valid_framerate_code(picture_rate))
     {
 		frame_rate = Y4M_RATIO_DBL(mpeg_framerate(picture_rate));
 	}
@@ -396,7 +396,7 @@ void ZAlphaStream::OutputSeqhdrInfo ()
     mjpeg_info ("Z format 0x%x, depth    : %d", z_format, z_depth);
     mjpeg_info ("Alpha depth   : %d", alpha_depth);
 
-	if( aspect_ratio <= mpeg_num_aspect_ratios[muxinto.mpeg-1] )
+	if (mpeg_valid_aspect_code(muxinto.mpeg, aspect_ratio))
 		str =  mpeg_aspect_code_definition(muxinto.mpeg,aspect_ratio);
 	else
 		str = "forbidden";
@@ -404,7 +404,7 @@ void ZAlphaStream::OutputSeqhdrInfo ()
 
     if (picture_rate == 0)
 		mjpeg_info( "Picture rate    : forbidden");
-    else if (picture_rate <= mpeg_num_framerates)
+    else if (mpeg_valid_framerate_code(picture_rate))
 		mjpeg_info( "Picture rate    : %2.3f frames/sec",
 					Y4M_RATIO_DBL(mpeg_framerate(picture_rate)) );
     else
