@@ -51,7 +51,7 @@ void variance(uint8_t *,int ,int, unsigned int *,unsigned int *);
 
 
 
-#if defined(HAVE_ASM_MMX) && defined(HAVE_ASM_NASM)
+#if defined(HAVE_ASM_MMX)
 #include "mmxsse/mmxsse_motion.h"
 #endif
 
@@ -152,11 +152,9 @@ void sub_mean_reduction( me_result_set *matchset,
 	int i,j;
 	int weight_sum;
 	int mean_weight;
-	int min_weight = 100000;
-	if( len == 0 )
+	if( len <= 1 )
 	{
-		*minweight_res = 100000;
-		matchset->len = 0;
+		*minweight_res = (len==0) ? 100000 : matches[0].weight;
 		return;
 	}
 
@@ -175,10 +173,6 @@ void sub_mean_reduction( me_result_set *matchset,
 		{
 			if( matches[i].weight <= mean_weight )
 			{
-				if( times == 1)
-				{
-					min_weight = matches[i].weight ;
-				}
 				matches[j] = matches[i];
 				++j;
 			}
@@ -278,7 +272,7 @@ STATIC int build_sub44_mests( me_result_set *sub44set,
 				sub44_mests[sub44_num_mests].x = i;
 				sub44_mests[sub44_num_mests].y = j;
 				sub44_mests[sub44_num_mests].weight = s1 + 
-					(intmax(intabs(i-i0),intabs(j-j0))<<1);
+					(intmax(abs(i-i0), abs(j-j0))<<1);
 				++sub44_num_mests;
 			}
 			s44orgblk += 1;
@@ -343,7 +337,7 @@ STATIC int build_sub22_mests( me_result_set *sub44set,
 			if( x <= ilim && y <= jlim )
 			{	
 				s = (*psad_sub22)( s22orgblk,s22blk,frowstride,fh)+
-					(intmax(intabs(x),intabs(y))<<3);
+					(intmax(abs(x), abs(y))<<3);
 				if( s < threshold )
 				{
 					me_result_s *mc = &sub22set->mests[sub22set->len];
@@ -407,7 +401,7 @@ STATIC void find_best_one_pel( me_result_set *sub22set,
 
 		matchrec = sub22set->mests[k];
 		orgblk = org + (i0+matchrec.x)+rowstride*(j0+matchrec.y);
-		penalty = (intabs(matchrec.x)+intabs(matchrec.y))<<3;
+		penalty = (abs(matchrec.x) + abs(matchrec.y))<<3;
 		for( i = 0; i < 4; ++i )
 		{
 			if( matchrec.x <= ilim && matchrec.y <= jlim )
@@ -499,7 +493,7 @@ STATIC int sad_01(uint8_t *blk1,uint8_t *blk2,int rowstride, int h)
 		{
 
 			v = ((unsigned int)(p1[i]+p1[i+1]+1)>>1) - p2[i];
-			s+=intabs(v);
+			s += abs(v);
 		}
 		p1+= rowstride;
 		p2+= rowstride;
@@ -523,7 +517,7 @@ STATIC int sad_10(uint8_t *blk1,uint8_t *blk2, int rowstride, int h)
 		for (i=0; i<16; i++)
 		{
 			v = ((unsigned int)(p1[i]+p1a[i]+1)>>1) - p2[i];
-			s+= intabs(v);
+			s += abs(v);
 		}
 		p1 = p1a;
 		p1a+= rowstride;
@@ -550,7 +544,7 @@ STATIC int sad_11(uint8_t *blk1,uint8_t *blk2, int rowstride, int h)
 		for (i=0; i<16; i++)
 		{
 			v = ((unsigned int)((p1[i]+p1[i+1])+(p1a[i]+p1a[i+1])+2)>>2) - p2[i];
-			s+=intabs(v);
+			s += abs(v);
 		}
 		p1 = p1a;
 		p1a+= rowstride;
@@ -996,7 +990,7 @@ void init_motion_search(void)
 	pbuild_sub44_mests = build_sub44_mests;
 	psubsample_image = subsample_image;
 
-#if defined(HAVE_ASM_MMX) && defined(HAVE_ASM_NASM)
+#if defined(HAVE_ASM_MMX)
 	enable_mmxsse_motion(cpucap);
 #endif
 #ifdef HAVE_ALTIVEC
